package dk.nsp.idsas.dao;

import dk.nsp.idsas.dao.entity.OrgBlurringEntity;
import org.springframework.jdbc.core.namedparam.NamedParameterJdbcTemplate;

import javax.sql.DataSource;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.sql.Timestamp;
import java.time.LocalDateTime;
import java.time.OffsetDateTime;
import java.time.format.DateTimeFormatter;
import java.time.format.DateTimeFormatterBuilder;
import java.util.HashMap;
import java.util.List;

public class OrgBlurringDao {
    private final NamedParameterJdbcTemplate jdbcTemplate;
    private final DateTimeFormatter formatter;

    public OrgBlurringDao(DataSource dataSource) {
        this.jdbcTemplate = new NamedParameterJdbcTemplate(dataSource);
        this.formatter = new DateTimeFormatterBuilder().append(DateTimeFormatter.ISO_OFFSET_DATE_TIME).toFormatter();
    }

    public void insert(OrgBlurringEntity orgBlurringEntity) {
        String query = "INSERT INTO organisation_blurring(organisation_id, organisation_id_type, creator_cvr, valid_from, valid_to, created_time)" +
                " VALUES (:organisation_id, :organisation_id_type, :creator_cvr, :valid_from, :valid_to, :created_time)";
        HashMap<String, Object> parameters = new HashMap<>();
        parameters.put("organisation_id", orgBlurringEntity.getOrganisationId());
        parameters.put("organisation_id_type", orgBlurringEntity.getOrganisationIdType());
        parameters.put("creator_cvr", orgBlurringEntity.getCreatorCvr());
        parameters.put("created_time", formatter.format(OffsetDateTime.now()));
        parameters.put("valid_from",  Timestamp.from(orgBlurringEntity.getValidFrom()));
        if (orgBlurringEntity.getValidTo() != null) {
            parameters.put("valid_to", Timestamp.from(orgBlurringEntity.getValidTo()));
        } else {
            parameters.put("valid_to", null);
        }

        jdbcTemplate.update(query, parameters);
    }

    public List<OrgBlurringEntity> getAllOrgBlurrings(String creatorCvr) {
        String query = "SELECT organisation_id, organisation_id_type, creator_cvr\n" +
                "FROM organisation_blurring\n" +
                "WHERE creator_cvr = :creator_cvr\n";

        HashMap<String, Object> parameters = new HashMap<>();
        parameters.put("creator_cvr", creatorCvr);
        return jdbcTemplate.query(query, parameters, (rs, rowNum) -> mapOrgBlurringEntityRow(rs));
    }

    public List<OrgBlurringEntity> getActiveOrgBlurrings(String organizationId, String organizationIdType) {
        String query = "SELECT organisation_id, organisation_id_type, creator_cvr\n" +
                "FROM organisation_blurring\n" +
                "WHERE organisation_id = :organisation_id\n" +
                "AND organisation_id_type = :organisation_id_type\n" +
                "AND (valid_to > :now OR valid_to IS NULL)\n" +
                "AND valid_from < :now;";

        HashMap<String, Object> parameters = new HashMap<>();
        parameters.put("organisation_id", organizationId);
        parameters.put("organisation_id_type", organizationIdType);
        parameters.put("now", LocalDateTime.now().toString());
        return jdbcTemplate.query(query, parameters, (rs, rowNum) -> mapOrgBlurringEntityRow(rs));
    }

    public List<OrgBlurringEntity> getAllActiveOrgBlurrings() {
        String query = "SELECT organisation_id, organisation_id_type, creator_cvr\n" +
                "FROM organisation_blurring\n" +
                "WHERE (valid_to > :now OR valid_to IS NULL)\n" +
                "AND valid_from < :now;";

        HashMap<String, Object> parameters = new HashMap<>();
        parameters.put("now", LocalDateTime.now().toString());
        return jdbcTemplate.query(query, parameters, (rs, rowNum) -> mapOrgBlurringEntityRow(rs));
    }

    public boolean orgBlurringExists(String organizationId, String organizationIdType) {
        String query = "SELECT EXISTS(\n" +
                "SELECT 1 FROM organisation_blurring\n" +
                "WHERE organisation_id = :organisation_id\n" +
                "AND organisation_id_type = :organisation_id_type\n" +
                "AND (valid_to > :now OR valid_to IS NULL)\n" +
                "AND valid_from < :now\n" +
                ");";

        HashMap<String, Object> parameters = new HashMap<>();
        parameters.put("organisation_id", organizationId);
        parameters.put("organisation_id_type", organizationIdType);
        parameters.put("now", LocalDateTime.now().toString());
        return Boolean.TRUE.equals(jdbcTemplate.queryForObject(query, parameters, Boolean.class));
    }

    public void removeOrgBlurring(String organizationId, String organizationIdType) {
        String query = "UPDATE organisation_blurring\n" +
                "SET valid_to = :now\n" +
                "WHERE organisation_id = :organisation_id\n" +
                "AND organisation_id_type = :organisation_id_type\n" +
                "AND valid_to IS NULL\n";

        HashMap<String, Object> parameters = new HashMap<>();
        parameters.put("organisation_id", organizationId);
        parameters.put("organisation_id_type", organizationIdType);
        parameters.put("now", LocalDateTime.now().toString());
        jdbcTemplate.update(query, parameters);
    }

    private static OrgBlurringEntity mapOrgBlurringEntityRow(ResultSet rs) throws SQLException {
        OrgBlurringEntity orgBlurringEntity = new OrgBlurringEntity();
        orgBlurringEntity.setOrganisationId(rs.getString("organisation_id"));
        orgBlurringEntity.setOrganisationIdType(rs.getString("organisation_id_type"));
        orgBlurringEntity.setCreatorCvr(rs.getString("creator_cvr"));
        return orgBlurringEntity;
    }
}
