package dk.nsp.idsas.security.impl;

import dk.nsp.idsas.security.IdsasActorProvider;
import dk.nsp.idsas.security.IdsasSecurityException;
import dk.nsp.idsas.security.IdsasActor;
import dk.nsp.idsas.util.ErrorMessages;
import dk.nsp.idsas.util.FaultCode;
import dk.sds.nsp.security.SecurityContext;
import org.apache.log4j.Logger;

import java.time.ZonedDateTime;
import java.util.LinkedList;
import java.util.List;
import java.util.function.Supplier;

public class IdsasActorProviderImpl implements IdsasActorProvider {
    private static final Logger logger = Logger.getLogger(IdsasActorProviderImpl.class);
    // Formålet med at bruge en Supplier er, at vi kan udsætte "getSecurityContext"
    // til vi er i selve ServletRequest-tråden.
    final private Supplier<SecurityContext> securitySupplier;

    public IdsasActorProviderImpl(Supplier<SecurityContext> securitySupplier) {
        this.securitySupplier = securitySupplier;
    }

    @Override
    public IdsasActor getActor() throws IdsasSecurityException {
        SecurityContext securityContext = securitySupplier.get();

        // Der skal være en sikkerhedsbillet
        if (!securityContext.getTicket().isPresent()) {
            throw new IdsasSecurityException(ErrorMessages.ticketIsMissing(), FaultCode.ACTOR_VALIDATION_ERROR, true);
        }

        // Sikkerhedsbilletten skal være valid
        validateTicket(securityContext);

        // Gennemløber de enkelte brugertyper
        List<IdsasActor> matchedActors = new LinkedList<>();
        List<String> causesList = new LinkedList<>();

        IdsasActor system = matchSystemActor(securityContext, causesList);
        if (system != null) {
            matchedActors.add(system);
        } else {
            logger.debug("Brugertype matcher ikke systembruger");
        }

        IdsasActor sundhedsfaglig = matchSundhedsfagligActor(securityContext, causesList);
        if (sundhedsfaglig != null) {
            matchedActors.add(sundhedsfaglig);
        } else {
            logger.debug("Brugertype matcher ikke sundhedsfaglig");
        }

        if (matchedActors.isEmpty()) {
            String msg = ErrorMessages.invalidUserType(String.join(", ", causesList));
            throw new IdsasSecurityException(msg, FaultCode.ACTOR_VALIDATION_ERROR, true);
        } else if (matchedActors.size() > 1) {
            String actors = matchedActors.stream().map(IdsasActor::getActorType).map(IdsasActor.IdsasActorType::getDisplayableText).reduce("", (a, b) -> a + "," + b);
            String msg = ErrorMessages.userTypeAmbiguous(actors);
            throw new IdsasSecurityException(msg, FaultCode.ACTOR_VALIDATION_ERROR, true);
        }

        return matchedActors.get(0);
    }

    private void validateTicket(SecurityContext securityContext) throws IdsasSecurityException {
        if (securityContext.getTicket().isPresent() && !securityContext.getTicket().get().isValid()) {

            // Den må ikke være udløbet
            if (securityContext.getTicket().get().getValidTo().isPresent()) {
                ZonedDateTime validTo = securityContext.getTicket().get().getValidTo().get();
                if (validTo.isBefore(ZonedDateTime.now())) {
                    throw new IdsasSecurityException(ErrorMessages.ticketIsExpiredError(), FaultCode.ACTOR_VALIDATION_ERROR, true);
                }
            }

            // Den må ikke først være gyldig senere
            if (securityContext.getTicket().get().getValidFrom().isPresent()) {
                ZonedDateTime validFrom = securityContext.getTicket().get().getValidFrom().get();
                if (validFrom.isAfter(ZonedDateTime.now())) {
                    throw new IdsasSecurityException(ErrorMessages.ticketIsNotYetValidError(), FaultCode.ACTOR_VALIDATION_ERROR, true);
                }
            }

            // Generel fejlbesked, hvis der er noget andet galt
            String audience = securityContext.getTicket().flatMap(SecurityContext.Ticket::getAudience).orElse("");
            String created = securityContext.getTicket()
                    .flatMap(SecurityContext.Ticket::getCreated)
                    .map(Object::toString)
                    .orElse("");

            throw new IdsasSecurityException(ErrorMessages.ticketIsNotValidError(audience, created), FaultCode.ACTOR_VALIDATION_ERROR, true);
        }
    }

    private IdsasActor matchSystemActor(SecurityContext securityContext, List<String> causesList) {
        if (securityContext.getActingUser().isPresent()) {
            causesList.add("Systembruger kan ikke være en acting user");
            return null;
        }

        if (securityContext.getPrincipalUser().isPresent()) {
            causesList.add("Systembruger kan ikke være en principal user");
            return null;
        }

        if (!securityContext.getOrganisation().isPresent()) {
            causesList.add("Systembruger skal have en organisation");
            return null;
        }

        String organisationIdentifier = securityContext.getOrganisation().get().getIdentifier();
        String organisationIdentifierType = securityContext.getOrganisation().get().getIdentifierFormat().name();
        String clientName = securityContext.getClient().flatMap(SecurityContext.Client::getName).orElse("");
        String clientKey = securityContext.getClient()
                .flatMap(SecurityContext.Client::getPersistentUniqueKey)
                .map(SecurityContext.PersistentUniqueKey::value)
                .orElse("");
        return IdsasActor.createSystem(organisationIdentifier, organisationIdentifierType, clientName, clientKey);
    }

    private IdsasActor matchSundhedsfagligActor(SecurityContext securityContext, List<String> causesList) {
        if (securityContext.getPrincipalUser().isPresent()) {
            causesList.add("Sundhedsfaglig kan ikke være en principal user");
            return null;
        }

        if (!securityContext.getActingUser().isPresent()) {
            causesList.add("Sundhedsfaglig skal have en acting user");
            return null;
        }

        if (!securityContext.getActingUser().get().getUserType().equals(
                SecurityContext.User.UserType.HealthcareProfessional)) {
            causesList.add("Sundhedsfaglig ActingUser skal have typen HealthcareProfessional");
            return null;
        }


        if (!securityContext.getOrganisation().isPresent()) {
            causesList.add("Sundhedsfaglig skal have en organisation");
            return null;
        }

        String organisationIdentifier = securityContext.getOrganisation().get().getIdentifier();
        String organisationIdentifierType = securityContext.getOrganisation().get().getIdentifierFormat().name();
        return IdsasActor.createSundhedsFaglig(organisationIdentifier, organisationIdentifierType);
    }
}
