package dk.nsp.idsas.security.impl;

import dk.nsp.idsas.security.IdsasActor;
import dk.nsp.idsas.security.IdsasActorValidator;
import dk.nsp.idsas.security.IdsasSecurityException;
import dk.nsp.idsas.util.FaultCode;
import dk.sds.nsp.security.SecurityContext;
import dk.nsp.idsas.util.ErrorMessages;

import java.util.function.Supplier;

public class IdsasActorValidatorImpl implements IdsasActorValidator {
    final private Supplier<SecurityContext> securitySupplier;

    public IdsasActorValidatorImpl(Supplier<SecurityContext> securitySupplier) {
        this.securitySupplier = securitySupplier;
    }

    public IdsasActor getActor(IdsasActor actor) throws IdsasSecurityException {
        switch (actor.getActorType()) {
            case SUNDHEDSFAGLIG:
                return validateSundhedsfaglig(actor);
            case SYSTEM:
                return validateSystem(actor);
        }
        return null; // Not reachable currently
    }

    private IdsasActor validateSystem(IdsasActor actor) throws IdsasSecurityException {
        SecurityContext securityContext = securitySupplier.get();

        if (actor.getOrganizationId().isEmpty()) {
            String msg = "Systembruger skal have en organisation med en identifier";
            throw new IdsasSecurityException(ErrorMessages.actorValidationError(
                    actor.getActorType().getDisplayableText(), msg),
                    FaultCode.ACTOR_VALIDATION_ERROR, true);
        }

        if (!securityContext.getOrganisation().get().getIdentifierFormat().equals(
                SecurityContext.Organisation.OrganisationIdentifierFormat.CVR)) {
            String msg = "Systembruger skal have en organisation af typen CVR";
            throw new IdsasSecurityException(ErrorMessages.actorValidationError(
                    actor.getActorType().getDisplayableText(), msg),
                    FaultCode.ACTOR_VALIDATION_ERROR, true);
        }

        return actor;
    }

    private IdsasActor validateSundhedsfaglig(IdsasActor actor) throws IdsasSecurityException {
        SecurityContext securityContext = securitySupplier.get();

        if (actor.getOrganizationId().isEmpty()) {
            String msg = "Sundhedsfaglig skal have en organisation med en identifier";
            throw new IdsasSecurityException(ErrorMessages.actorValidationError(
                    actor.getActorType().getDisplayableText(), msg),
                    FaultCode.ACTOR_VALIDATION_ERROR, true);
        }

        if (!securityContext.getOrganisation().get().getIdentifierFormat().equals(
                SecurityContext.Organisation.OrganisationIdentifierFormat.CVR)) {
            String msg = "Sundhedsfaglig skal have en organisation af typen CVR";
            throw new IdsasSecurityException(ErrorMessages.actorValidationError(
                    actor.getActorType().getDisplayableText(), msg),
                    FaultCode.ACTOR_VALIDATION_ERROR, true);
        }

        if (securityContext.getActingUser().get().getIdentifier().isEmpty()) {
            String msg = "Sundhedsfaglig ActingUser skal have en identifier";
            throw new IdsasSecurityException(ErrorMessages.actorValidationError(
                    actor.getActorType().getDisplayableText(), msg),
                    FaultCode.ACTOR_VALIDATION_ERROR, true);
        }

        if (!securityContext.getActingUser().get().getIdentifierFormat().equals(
                SecurityContext.User.PersonIdentifierFormat.CPR)) {
            String msg = "Sundhedsfaglig ActingUser skal have en identifier af typen CPR";
            throw new IdsasSecurityException(ErrorMessages.actorValidationError(
                    actor.getActorType().getDisplayableText(), msg),
                    FaultCode.ACTOR_VALIDATION_ERROR, true);
        }

        return actor;
    }
}
