package dk.nsp.idsas.util;

import dk.sds.nsp.audit.Audit;
import dk.sds.nsp.audit.AuditBuilder;
import dk.sds.nsp.audit.AuditBuilder.TypeOfInformation;

import java.time.Instant;
import java.util.Base64;
import java.util.function.Supplier;

public class AuditLogger {
    private static final String COMPONENT = "IDSAS";
    private static final String CREATE_BLURRING_CONTEXT = "createBlurring";
    private static final String GET_CURRENT_SALT_CONTEXT = "getCurrentSalt";
    private static final String GET_BLURRED_ORGANISATIONS_CONTEXT = "getBlurredOrganisations";
    private final Supplier<AuditBuilder> auditBuilderSupplier;

    public AuditLogger() {
        this(Audit::createAuditBuilder);
    }

    public AuditLogger(Supplier<AuditBuilder> auditBuilderSupplier) {
        this.auditBuilderSupplier = auditBuilderSupplier;
    }

    public void logCreateBlurring(String patientId, String patientIdSource, String hashedPatientId, String actorId, String actorIdSource, Instant endDateTime) {
        AuditBuilder auditBuilder = auditBuilderSupplier.get();
        auditLog(auditBuilder, CREATE_BLURRING_CONTEXT, Type.PATIENT_ID, patientId);
        auditLog(auditBuilder, CREATE_BLURRING_CONTEXT, Type.PATIENT_ID_SOURCE, patientIdSource);
        auditLog(auditBuilder, CREATE_BLURRING_CONTEXT, Type.ACTOR_ID, actorId);
        auditLog(auditBuilder, CREATE_BLURRING_CONTEXT, Type.ACTOR_ID_SOURCE, actorIdSource);
        auditLog(auditBuilder, CREATE_BLURRING_CONTEXT, Type.HASHED_PATIENT_ID, hashedPatientId);
        auditLog(auditBuilder, CREATE_BLURRING_CONTEXT, Type.EXPIRY_DATE_TIME, endDateTime);
    }

    public void logGetCurrentSalt(String actorId, String actorIdSource, byte[] salt) {
        AuditBuilder auditBuilder = auditBuilderSupplier.get();
        auditLog(auditBuilder, GET_CURRENT_SALT_CONTEXT, Type.ACTOR_ID, actorId);
        auditLog(auditBuilder, GET_CURRENT_SALT_CONTEXT, Type.ACTOR_ID_SOURCE, actorIdSource);

        String base64EncodedSalt = Base64.getEncoder().encodeToString(salt);
        auditLog(auditBuilder, GET_CURRENT_SALT_CONTEXT, Type.SALT, base64EncodedSalt);
    }

    public void logGetBlurredOrganisations(String patientId, String patientIdSource) {
        AuditBuilder auditBuilder = auditBuilderSupplier.get();
        auditLog(auditBuilder, GET_BLURRED_ORGANISATIONS_CONTEXT, Type.PATIENT_ID, patientId);
        auditLog(auditBuilder, GET_BLURRED_ORGANISATIONS_CONTEXT, Type.PATIENT_ID_SOURCE, patientIdSource);
    }

    private void auditLog(AuditBuilder auditBuilder, String context, Type type, Object value) {
        auditBuilder.addAuditInformation(COMPONENT, context, type.typeOfInformation, type.value, value);
    }

    private enum Type {
        PATIENT_ID("patient-id", TypeOfInformation.RegularPersonalInformation),
        PATIENT_ID_SOURCE("patient-id-source", TypeOfInformation.RegularPersonalInformation),
        ACTOR_ID("actor-id", TypeOfInformation.NonPersonalInformation),
        ACTOR_ID_SOURCE("actor-id-source", TypeOfInformation.NonPersonalInformation),
        HASHED_PATIENT_ID("hashed-patient-id", TypeOfInformation.SensitivePersonalInformation),
        EXPIRY_DATE_TIME("expiry-date-time", TypeOfInformation.NonPersonalInformation),
        SALT("base64-encoded-salt", TypeOfInformation.NonPersonalInformation)
        ;

        private final String value;
        private final TypeOfInformation typeOfInformation;

        Type(String value, AuditBuilder.TypeOfInformation typeOfInformation) {
            this.value = value;
            this.typeOfInformation = typeOfInformation;
        }
    }
}
