package dk.nsp.idsas.dao;

import dk.nsp.idsas.dao.entity.OrgBlurringEntity;
import org.junit.Before;
import org.junit.Test;
import org.springframework.beans.factory.annotation.Autowired;

import javax.sql.DataSource;
import java.time.*;
import java.util.List;

import static org.junit.Assert.*;

public class OrgBlurringDaoTest extends AbstractDaoTest {
    @Autowired
    DataSource dataSource;

    private OrgBlurringDao orgBlurringDao;

    private final OrgBlurringEntity TEST_BLURRING_1 =
            new OrgBlurringEntity("1111", "SOR", "12345678",
                    instantFrom(2022, 1, 1, 0, 0), instantFrom(3000, 1, 1, 0, 0));

    private final OrgBlurringEntity TEST_BLURRING_6 =
            new OrgBlurringEntity("6666", "SOR", "22221111",
                    instantFrom(2022, 1, 1, 0, 0), instantFrom(2500, 1, 1, 0, 0));

    private final OrgBlurringEntity TEST_BLURRING_NO_EXPIRATION =
            new OrgBlurringEntity("1122", "SOR", "33456789",
                    instantFrom(2022, 1, 1, 0, 0), null);

    private final OrgBlurringEntity TEST_BLURRING_EXPIRED =
            new OrgBlurringEntity("5555", "SOR", "11112222",
                    instantFrom(2020, 1, 1, 0, 0), instantFrom(2021, 1, 1, 0, 0));

    @Before
    public void setUp() {
        orgBlurringDao = new OrgBlurringDao(dataSource);
    }

    @Test
    public void insert_WhenCalled_InsertsBlurring() {
        // Arrange
        OrgBlurringEntity blurringEntity = new OrgBlurringEntity("2222", "CVR", "12345678",
                instantFrom(2022, 1, 1, 0, 0), instantFrom(3000, 1, 1, 0, 0));

        // Act
        orgBlurringDao.insert(blurringEntity);

        // Assert
        List<OrgBlurringEntity> blurrings = orgBlurringDao.getAllOrgBlurrings("12345678");
        assertEquals(2, blurrings.size());
    }

    @Test
    public void insert_WhenCalled_InsertsBlurringWithCorrectValues() {
        // Arrange
        OrgBlurringEntity blurringEntity = new OrgBlurringEntity("9876", "CVR", "99887766",
                instantFrom(2022, 1, 1, 0, 0), instantFrom(3001, 2, 2, 0, 0));

        // Act
        orgBlurringDao.insert(blurringEntity);

        // Assert
        List<OrgBlurringEntity> blurrings = orgBlurringDao.getAllOrgBlurrings("99887766");
        assertEquals(1, blurrings.size());
        OrgBlurringEntity returnedBlurring = blurrings.get(0);
        assertEquals(blurringEntity.getOrganisationId(), returnedBlurring.getOrganisationId());
        assertEquals(blurringEntity.getOrganisationIdType(), returnedBlurring.getOrganisationIdType());
    }

    @Test
    public void getAllOrgBlurrings_WhenOneMatchingActiveBlurringIsFound_ReturnsBlurring() {
        // Act
        List<OrgBlurringEntity> blurrings = orgBlurringDao.getAllOrgBlurrings("12345678");

        // Assert
        assertEquals(1, blurrings.size());
        OrgBlurringEntity returnedBlurring = blurrings.get(0);
        assertEquals(TEST_BLURRING_1.getOrganisationId(), returnedBlurring.getOrganisationId());
        assertEquals(TEST_BLURRING_1.getOrganisationIdType(), returnedBlurring.getOrganisationIdType());
    }

    @Test
    public void getAllOrgBlurrings_WhenOneActiveWithoutExpirationIsFound_ReturnsBlurring() {
        // Act
        List<OrgBlurringEntity> blurrings = orgBlurringDao.getAllOrgBlurrings("23456789");

        // Assert
        assertEquals(1, blurrings.size());
        OrgBlurringEntity returnedBlurring = blurrings.get(0);
        assertEquals(TEST_BLURRING_1.getOrganisationId(), returnedBlurring.getOrganisationId());
        assertEquals(TEST_BLURRING_1.getOrganisationIdType(), returnedBlurring.getOrganisationIdType());
    }

    @Test
    public void getAllOrgBlurrings_WhenNoMatchingBlurringIsFound_ReturnsEmptyList() {
        // Act
        List<OrgBlurringEntity> blurrings = orgBlurringDao.getAllOrgBlurrings("does not exist");

        // Assert
        assertEquals(0, blurrings.size());
    }

    @Test
    public void getAllOrgBlurrings_WhenOneMatchingExpiredBlurringIsFound_ReturnsBlurring() {
        // Act
        List<OrgBlurringEntity> blurrings = orgBlurringDao.getAllOrgBlurrings("11112222");

        // Assert
        assertEquals(1, blurrings.size());
        OrgBlurringEntity returnedBlurring = blurrings.get(0);
        assertEquals(TEST_BLURRING_EXPIRED.getOrganisationId(), returnedBlurring.getOrganisationId());
        assertEquals(TEST_BLURRING_EXPIRED.getOrganisationIdType(), returnedBlurring.getOrganisationIdType());
    }

    @Test
    public void getAllOrgBlurrings_WhenTwoMatchingActiveBlurringIsFound_ReturnsBlurring() {
        // Act
        List<OrgBlurringEntity> blurrings = orgBlurringDao.getAllOrgBlurrings("22221111");

        // Assert
        assertEquals(2, blurrings.size());
        OrgBlurringEntity returnedBlurring = blurrings.get(0);
        assertEquals(TEST_BLURRING_6.getOrganisationId(), returnedBlurring.getOrganisationId());
        assertEquals(TEST_BLURRING_6.getOrganisationIdType(), returnedBlurring.getOrganisationIdType());
    }

    @Test
    public void getActiveOrgBlurrings_WhenTwoMatchingActiveBlurringIsFound_ReturnsBlurring() {
        // Act
        List<OrgBlurringEntity> blurrings = orgBlurringDao.getActiveOrgBlurrings("1111", "SOR");

        // Assert
        assertEquals(2, blurrings.size());
        OrgBlurringEntity returnedBlurring = blurrings.get(0);
        assertEquals(TEST_BLURRING_1.getOrganisationId(), returnedBlurring.getOrganisationId());
        assertEquals(TEST_BLURRING_1.getOrganisationIdType(), returnedBlurring.getOrganisationIdType());
    }

    @Test
    public void getActiveOrgBlurrings_WhenOneMatchingActiveBlurringIsFound_ReturnsBlurring() {
        // Act
        List<OrgBlurringEntity> blurrings = orgBlurringDao.getActiveOrgBlurrings("1122", "SOR");

        // Assert
        assertEquals(1, blurrings.size());
        OrgBlurringEntity returnedBlurring = blurrings.get(0);
        assertEquals(TEST_BLURRING_NO_EXPIRATION.getOrganisationId(), returnedBlurring.getOrganisationId());
        assertEquals(TEST_BLURRING_NO_EXPIRATION.getOrganisationIdType(), returnedBlurring.getOrganisationIdType());
    }

    @Test
    public void getActiveOrgBlurrings_WhenNoMatchingActiveBlurringIsFound_ReturnsEmptyList() {
        // Act
        List<OrgBlurringEntity> blurrings = orgBlurringDao.getActiveOrgBlurrings("does not exist", "SOR");

        // Assert
        assertEquals(0, blurrings.size());
    }

    @Test
    public void getAllActiveOrgBlurrings_returnsThemAll() {
        // Act
        List<OrgBlurringEntity> blurrings = orgBlurringDao.getAllActiveOrgBlurrings();

        // Assert
        assertEquals(8, blurrings.size());
    }

    @Test
    public void orgBlurringExists_WhenBlurringExists_ReturnsTrue() {
        // Act
        boolean exists = orgBlurringDao.orgBlurringExists("1111", "SOR");

        // Assert
        assertTrue(exists);
    }

    @Test
    public void orgBlurringExists_WhenBlurringDoesNotExist_ReturnsFalse() {
        // Act
        boolean exists = orgBlurringDao.orgBlurringExists("does not exist", "SOR");

        // Assert
        assertFalse(exists);
    }

    @Test
    public void removeOrgBlurring_RemovesExistingBlurring_ItsNoLongerActive() {
        // Act
        List<OrgBlurringEntity> before = orgBlurringDao.getActiveOrgBlurrings("3333", "SOR");
        orgBlurringDao.removeOrgBlurring("3333", "SOR");

        // Assert
        List<OrgBlurringEntity> after = orgBlurringDao.getActiveOrgBlurrings("3333", "SOR");
        assertEquals(1, before.size());
        assertEquals(0, after.size());

    }

    @Test
    public void removeOrgBlurring_RemovesExistingBlurring_ItsStillInAllOrgBlurrings() {
        // Act
        List<OrgBlurringEntity> before = orgBlurringDao.getAllOrgBlurrings("12344321");
        orgBlurringDao.removeOrgBlurring("3333", "SOR");

        // Assert
        List<OrgBlurringEntity> after = orgBlurringDao.getAllOrgBlurrings("12344321");
        assertEquals(1, before.size());
        assertEquals(1, after.size());
    }

    private static Instant instantFrom(int year, int month, int dayOfMonth, int hour, int minute) {
        return LocalDateTime.of(year, month, dayOfMonth, hour, minute).atZone(ZoneId.systemDefault()).toInstant();
    }
}
