package dk.nsp.idsas.security;

import dk.nsp.idsas.security.impl.IdsasActorProviderImpl;
import dk.nsp.idsas.util.ErrorMessages;
import dk.sds.nsp.security.SecurityContext;
import org.junit.Before;
import org.junit.Test;

import java.time.ZonedDateTime;
import java.util.Optional;

import static junit.framework.TestCase.assertEquals;
import static org.junit.Assert.assertThrows;
import static org.junit.Assert.assertTrue;
import static org.mockito.Mockito.mock;
import static org.mockito.Mockito.when;

public class IdsasActorProviderTest {

    private SecurityContext securityContext;
    private SecurityContext.Ticket ticket;
    private SecurityContext.User principalUser;
    private SecurityContext.User actingUser;
    private SecurityContext.Organisation organisation;

    @Before
    public void setup() {
        securityContext = mock(SecurityContext.class);
        ticket = mock(SecurityContext.Ticket.class);
        principalUser = mock(SecurityContext.User.class);
        actingUser = mock(SecurityContext.User.class);
        organisation = mock(SecurityContext.Organisation.class);

        when(securityContext.getTicket()).thenReturn(Optional.of(ticket));
        when(ticket.isValid()).thenReturn(true);
    }

    @Test
    public void matchSystembruger() throws IdsasSecurityException {
        // Arrange
        IdsasActorProvider provider = new IdsasActorProviderImpl(() -> securityContext);
        when(securityContext.getOrganisation()).thenReturn(Optional.of(organisation));
        when(organisation.getIdentifierFormat()).thenReturn(SecurityContext.Organisation.OrganisationIdentifierFormat.CVR);
        when(organisation.getIdentifier()).thenReturn("123");

        // Act
        IdsasActor actor = provider.getActor();

        // Assert
        assertEquals(IdsasActor.IdsasActorType.SYSTEM, actor.getActorType());
        assertEquals("123", actor.getOrganizationId());
    }

    @Test
    public void matchSystembruger_principalUserGiven_noMatch() {
        // Arrange
        IdsasActorProvider provider = new IdsasActorProviderImpl(() -> securityContext);
        when(securityContext.getOrganisation()).thenReturn(Optional.of(organisation));
        when(organisation.getIdentifierFormat()).thenReturn(SecurityContext.Organisation.OrganisationIdentifierFormat.CVR);
        when(organisation.getIdentifier()).thenReturn("123");
        when(securityContext.getPrincipalUser()).thenReturn(Optional.of(principalUser));

        // Act
        IdsasSecurityException e = assertThrows(IdsasSecurityException.class, provider::getActor);

        // Assert
        assertTrue(e.getMessage().contains("Brugertypen kunne ikke bestemmes ud fra den medsendte sikkerhedsbillet"));
    }

    @Test
    public void matchSystembruger_orgMissing_noMatch() {
        // Arrange
        IdsasActorProvider provider = new IdsasActorProviderImpl(() -> securityContext);
        when(organisation.getIdentifierFormat()).thenReturn(SecurityContext.Organisation.OrganisationIdentifierFormat.CVR);
        when(organisation.getIdentifier()).thenReturn("123");

        // Act
        IdsasSecurityException e = assertThrows(IdsasSecurityException.class, provider::getActor);

        // Assert
        assertTrue(e.getMessage().contains("Brugertypen kunne ikke bestemmes ud fra den medsendte sikkerhedsbillet"));
    }

    @Test
    public void matchSundhedsfaglig() throws IdsasSecurityException {
        // Arrange
        IdsasActorProvider provider = new IdsasActorProviderImpl(() -> securityContext);
        when(principalUser.getIdentifier()).thenReturn("123");
        when(principalUser.getIdentifierFormat()).thenReturn(SecurityContext.User.PersonIdentifierFormat.CPR);
        when(securityContext.getActingUser()).thenReturn(Optional.of(actingUser));
        when(actingUser.getUserType()).thenReturn(SecurityContext.User.UserType.HealthcareProfessional);
        when(securityContext.getOrganisation()).thenReturn(Optional.of(organisation));
        when(organisation.getIdentifierFormat()).thenReturn(SecurityContext.Organisation.OrganisationIdentifierFormat.CVR);
        when(organisation.getIdentifier()).thenReturn("123");

        // Act
        IdsasActor actor = provider.getActor();

        // Assert
        assertEquals(actor.getActorType(), IdsasActor.IdsasActorType.SUNDHEDSFAGLIG);
    }

    @Test
    public void matchSundhedsfaglig_actingUserTypeWring_noMatch() {
        // Arrange
        IdsasActorProvider provider = new IdsasActorProviderImpl(() -> securityContext);
        when(principalUser.getIdentifier()).thenReturn("123");
        when(principalUser.getIdentifierFormat()).thenReturn(SecurityContext.User.PersonIdentifierFormat.CPR);
        when(securityContext.getActingUser()).thenReturn(Optional.of(actingUser));
        when(actingUser.getUserType()).thenReturn(SecurityContext.User.UserType.Citizen); // Wrong type
        when(securityContext.getOrganisation()).thenReturn(Optional.of(organisation));
        when(organisation.getIdentifierFormat()).thenReturn(SecurityContext.Organisation.OrganisationIdentifierFormat.CVR);
        when(organisation.getIdentifier()).thenReturn("123");


        // Act
        IdsasSecurityException e = assertThrows(IdsasSecurityException.class, provider::getActor);

        // Assert
        assertTrue(e.getMessage().contains("Brugertypen kunne ikke bestemmes ud fra den medsendte sikkerhedsbillet"));
    }


    @Test
    public void matchSundhedsfaglig_orgMissing_noMatch() {
        // Arrange
        IdsasActorProvider provider = new IdsasActorProviderImpl(() -> securityContext);
        when(principalUser.getIdentifier()).thenReturn("123");
        when(principalUser.getIdentifierFormat()).thenReturn(SecurityContext.User.PersonIdentifierFormat.CPR);
        when(securityContext.getActingUser()).thenReturn(Optional.of(actingUser));
        when(actingUser.getUserType()).thenReturn(SecurityContext.User.UserType.HealthcareProfessional);
        when(organisation.getIdentifierFormat()).thenReturn(SecurityContext.Organisation.OrganisationIdentifierFormat.CVR);
        when(organisation.getIdentifier()).thenReturn("123");

        // Act
        IdsasSecurityException e = assertThrows(IdsasSecurityException.class, provider::getActor);

        // Assert
        assertTrue(e.getMessage().contains("Brugertypen kunne ikke bestemmes ud fra den medsendte sikkerhedsbillet"));
    }

    @Test
    public void ticketIsNotValid() {
        // Arrange
        SecurityContext securityContext = mock(SecurityContext.class);
        SecurityContext.Ticket ticket = mock(SecurityContext.Ticket.class);
        when(ticket.getValidTo()).thenReturn(Optional.of(ZonedDateTime.now().plusYears(1)));
        when(ticket.getValidFrom()).thenReturn(Optional.of(ZonedDateTime.now().minusYears(1)));
        when(securityContext.getTicket()).thenReturn(Optional.of(ticket));
        IdsasActorProvider provider = new IdsasActorProviderImpl(() -> securityContext);

        // Act
        IdsasSecurityException e = assertThrows(IdsasSecurityException.class, provider::getActor);

        // Assert
        String audience = securityContext.getTicket().flatMap(SecurityContext.Ticket::getAudience).orElse("");
        String created = securityContext.getTicket()
                .flatMap(SecurityContext.Ticket::getCreated)
                .map(Object::toString)
                .orElse("");
        assertEquals(ErrorMessages.ticketIsNotValidError(audience, created), e.getMessage());
    }

    @Test
    public void ticketIsExpired() {
        // Arrange
        SecurityContext securityContext = mock(SecurityContext.class);
        SecurityContext.Ticket ticket = mock(SecurityContext.Ticket.class);
        when(ticket.getValidTo()).thenReturn(Optional.of(ZonedDateTime.now().minusDays(1)));
        when(securityContext.getTicket()).thenReturn(Optional.of(ticket));
        IdsasActorProvider provider = new IdsasActorProviderImpl(() -> securityContext);

        // Act
        IdsasSecurityException e = assertThrows(IdsasSecurityException.class, provider::getActor);

        // Assert
        assertEquals(ErrorMessages.ticketIsExpiredError(), e.getMessage());
    }

    @Test
    public void tickedIsNotYetValid() {
        // Arrange
        SecurityContext securityContext = mock(SecurityContext.class);
        SecurityContext.Ticket ticket = mock(SecurityContext.Ticket.class);
        when(ticket.getValidTo()).thenReturn(Optional.of(ZonedDateTime.now().plusYears(1)));
        when(ticket.getValidFrom()).thenReturn(Optional.of(ZonedDateTime.now().plusDays(1)));
        when(securityContext.getTicket()).thenReturn(Optional.of(ticket));
        IdsasActorProvider provider = new IdsasActorProviderImpl(() -> securityContext);

        // Act
        IdsasSecurityException e = assertThrows(IdsasSecurityException.class, provider::getActor);

        // Assert
        assertEquals(ErrorMessages.ticketIsNotYetValidError(), e.getMessage());
    }

    @Test
    public void ticketIsMissing() {
        // Arrange
        SecurityContext securityContext = mock(SecurityContext.class);
        when(securityContext.getTicket()).thenReturn(Optional.empty());
        IdsasActorProvider provider = new IdsasActorProviderImpl(() -> securityContext);

        // Act
        IdsasSecurityException e = assertThrows(IdsasSecurityException.class, provider::getActor);

        // Assert
        assertEquals(ErrorMessages.ticketIsMissing(), e.getMessage());
    }
}
