package dk.nsp.idsas.security;

import dk.nsp.idsas.security.impl.IdsasActorValidatorImpl;
import dk.sds.nsp.security.SecurityContext;
import org.junit.Before;
import org.junit.Test;
import org.mockito.Mockito;

import java.util.Optional;

import static org.junit.Assert.assertTrue;
import static org.junit.Assert.fail;
import static org.mockito.Mockito.when;

public class IdsasActorValidatorTest {
    private IdsasActorValidatorImpl validator;
    private SecurityContext securityContext;
    private IdsasActor actor;
    private SecurityContext.Organisation organisation;
    private SecurityContext.User actingUser;

    @Before
    public void setup() {
        securityContext = Mockito.mock(SecurityContext.class);
        actor = Mockito.mock(IdsasActor.class);
        organisation = Mockito.mock(SecurityContext.Organisation.class);
        actingUser = Mockito.mock(SecurityContext.User.class);

        validator = new IdsasActorValidatorImpl(() -> securityContext);
    }

    @Test
    public void validateSystem() throws IdsasSecurityException {
        when(actor.getOrganizationId()).thenReturn("12345678");
        when(actor.getActorType()).thenReturn(IdsasActor.IdsasActorType.SYSTEM);
        when(securityContext.getOrganisation()).thenReturn(Optional.of(organisation));
        when(organisation.getIdentifierFormat()).thenReturn(SecurityContext.Organisation.OrganisationIdentifierFormat.CVR);

        validator.getActor(actor);
    }

    @Test
    public void validateSystemNoOrgId() {
        when(actor.getOrganizationId()).thenReturn("");
        when(actor.getActorType()).thenReturn(IdsasActor.IdsasActorType.SYSTEM);
        when(securityContext.getOrganisation()).thenReturn(Optional.of(organisation));
        when(organisation.getIdentifierFormat()).thenReturn(SecurityContext.Organisation.OrganisationIdentifierFormat.CVR);
        try {
            validator.getActor(actor);
            fail("Should throw exception");
        } catch (IdsasSecurityException e) {
            assertTrue(e.getMessage().contains("Systembruger skal have en organisation med en identifier"));
        }
    }

    @Test
    public void validateSundhedsfaglig() throws IdsasSecurityException {
        when(actor.getOrganizationId()).thenReturn("12345678");
        when(actor.getActorType()).thenReturn(IdsasActor.IdsasActorType.SUNDHEDSFAGLIG);
        when(securityContext.getOrganisation()).thenReturn(Optional.of(organisation));
        when(securityContext.getActingUser()).thenReturn(Optional.of(actingUser));
        when(organisation.getIdentifierFormat()).thenReturn(SecurityContext.Organisation.OrganisationIdentifierFormat.CVR);
        when(actingUser.getIdentifier()).thenReturn("123");
        when(actingUser.getIdentifierFormat()).thenReturn(SecurityContext.User.PersonIdentifierFormat.CPR);

        validator.getActor(actor);
    }

    @Test
    public void validateSundhedsfagligNoOrgId() {
        when(actor.getOrganizationId()).thenReturn("");
        when(actor.getActorType()).thenReturn(IdsasActor.IdsasActorType.SUNDHEDSFAGLIG);
        when(securityContext.getOrganisation()).thenReturn(Optional.of(organisation));
        when(securityContext.getActingUser()).thenReturn(Optional.of(actingUser));
        when(organisation.getIdentifierFormat()).thenReturn(SecurityContext.Organisation.OrganisationIdentifierFormat.CVR);
        when(actingUser.getIdentifier()).thenReturn("123");
        when(actingUser.getIdentifierFormat()).thenReturn(SecurityContext.User.PersonIdentifierFormat.CPR);
        try {
            validator.getActor(actor);
            fail("Should throw exception");
        } catch (IdsasSecurityException e) {
            assertTrue(e.getMessage().contains("Sundhedsfaglig skal have en organisation med en identifier"));
        }
    }

    // "Sundhedsfaglig ActingUser skal have en identifier"
    @Test
    public void validateSundhedsfagligActingUserHasIdentifier() {
        when(actor.getOrganizationId()).thenReturn("1234");
        when(actor.getActorType()).thenReturn(IdsasActor.IdsasActorType.SUNDHEDSFAGLIG);
        when(securityContext.getOrganisation()).thenReturn(Optional.of(organisation));
        when(securityContext.getActingUser()).thenReturn(Optional.of(actingUser));
        when(organisation.getIdentifierFormat()).thenReturn(SecurityContext.Organisation.OrganisationIdentifierFormat.CVR);
        when(actingUser.getIdentifier()).thenReturn("");
        when(actingUser.getIdentifierFormat()).thenReturn(SecurityContext.User.PersonIdentifierFormat.CPR);
        try {
            validator.getActor(actor);
            fail("Should throw exception");
        } catch (IdsasSecurityException e) {
            assertTrue(e.getMessage().contains("Sundhedsfaglig ActingUser skal have en identifier"));
        }
    }

    @Test
    public void validateSundhedsfagligActingUserHasCprIdentifier() {
        when(actor.getOrganizationId()).thenReturn("1234");
        when(actor.getActorType()).thenReturn(IdsasActor.IdsasActorType.SUNDHEDSFAGLIG);
        when(securityContext.getOrganisation()).thenReturn(Optional.of(organisation));
        when(securityContext.getActingUser()).thenReturn(Optional.of(actingUser));
        when(organisation.getIdentifierFormat()).thenReturn(SecurityContext.Organisation.OrganisationIdentifierFormat.CVR);
        when(actingUser.getIdentifier()).thenReturn("");
        when(actingUser.getIdentifierFormat()).thenReturn(SecurityContext.User.PersonIdentifierFormat.CPR);
        try {
            validator.getActor(actor);
            fail("Should throw exception");
        } catch (IdsasSecurityException e) {
            assertTrue(e.getMessage().contains("Sundhedsfaglig ActingUser skal have en identifier"));
        }
    }

}