package dk.nsp.idsas.test._20240124;

import dk.nsp.idsas.test.idptest.SOAPOperation;
import dk.nsp.idsas.test.idptest.ServiceCaller;
import dk.nsp.idsas.test.setup.TestSetup;
import dk.nsp.idsas.test.util.TestIdentities;
import dk.nsp.idsas.util.ErrorMessages;
import dk.nsp.test.idp.model.Identity;
import dk.sundhedsdatastyrelsen.identitetssloering._2024._01._24.*;
import io.cucumber.java.en.And;
import io.cucumber.java.en.Then;
import io.cucumber.java.en.When;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;

import javax.xml.bind.JAXBException;
import javax.xml.datatype.DatatypeConfigurationException;
import java.net.URI;
import java.net.URISyntaxException;
import java.time.LocalDateTime;
import java.util.List;
import java.util.stream.Collectors;

import static dk.nsp.idsas.test.util.Util.toXmlGregorian;
import static org.junit.Assert.*;

public class BlurringStepDefinitions {

    @Value("${idsas.registration.endpoint}" + "/20240124/registration")
    private String idsasRegistrationUrl;

    @Value("${idsas.lookup.endpoint}" + "/20240124/lookup")
    private String idsasLookupUrl;

    @Value("${idsas.allowed.client.system.names}")
    private String[] clientSystemName;

    @Autowired
    private TestSetup testSetup;

    private String response = "";

    @When("en {string} opretter en slørring for en borger med CPR {string}")
    public void enOpretterEnSlørringForEnBorgerMedCPR(String brugertype, String cpr) throws DatatypeConfigurationException {
        createBlurring(brugertype, cpr, LocalDateTime.now().plusDays(1));
    }


    @And("der laves lookup på {string}, med clientName {string}, for at se om der er en slørring")
    public void derLavesLookupPåMedClientNameForAtSemOmDerErEnSlørring(String cpr, String clientName) {
        try {
            lookupBlurring(cpr, clientName);
        } catch (Exception e) {
            response = e.getMessage();
        }
    }

    @Then("så fejler lookup med fejl på clientName {string}")
    public void såFejlerLookupMedFejlPåClientName(String clientName) {
        String names = String.join(", ", clientSystemName);
        String expectedError = ErrorMessages.clientSystemNameError(clientName, names);
        assertTrue(response.contains(expectedError.substring(0, 100)));
    }

    @Then("så bliver slørringen oprettet")
    public void såBliverSlørringenOprettet() {
        assertEquals("OK", response);
    }

    @Then("så fejler oprettelsen af slørringen med forkert brugertype")
    public void såFejlerOprettelsenAfSlørringen() {
        assertTrue(response.contains(ErrorMessages.invalidUserType("")));
    }

    @When("en {string} opretter en slørring for en borger med CPR {string}, som udløb i går")
    public void enOpretterEnSlørringForEnBorgerMedCPRSomUdløberIGår(String brugertype, String cpr) throws DatatypeConfigurationException {
        // Sløringen udløber i går
        createBlurring(brugertype, cpr, LocalDateTime.now().minusDays(1));
    }

    @Then("så har {string} ikke længere aktuelle slørring over for det cvr-nummer")
    public void såHarIkkeLængereAktuelleSlørringOverForDetCvrNummer(String cpr) throws JAXBException, URISyntaxException {
        List<String> result = lookupBlurring(cpr, "DRG");
        String lookupTestIdentityCVR = "98021838";
        assertFalse(result.contains(lookupTestIdentityCVR));
    }

    @Then("så fejler oprettelsen af slørringen med ugyldigt CPR {string}")
    public void såFejlerOprettelsenAfSlørringenMedUgyldigtCpr(String cpr) {
        assertTrue(response, response.contains("CPR '"+cpr+"' er ikke tilknyttet en borger"));
    }

    @When("en {string} opretter en sløring der udløber for langt fremme i tiden")
    public void enBrugerOpretterEnSløringDerLiggerMereEndToÅrFremmeITiden(String userType) throws DatatypeConfigurationException {
        // Sløringen udløber om to år og én dag
        createBlurring(userType, "0102031234", LocalDateTime.now().plusYears(2).plusDays(1));
    }

    @Then("så fejler oprettelsen af slørringen med ugyldig slutdato")
    public void såFejlerOprettelsenAfSlørringenMedUgyldigSlutdato() {
        assertTrue(response.contains("Slutdatoen for en sløring må ikke ligge mere end"));
    }

    @When("en {string} opretter en sløring der udløber lige inden 30 dage er gået")
    public void enBrugerOpretterEnSløringDerUdløberOmLigeIndenToÅrErGået(String userType) throws DatatypeConfigurationException {
        // Sløringen ligger lige inden for to år
        createBlurring(userType, "0102031234", LocalDateTime.now().plusDays(29));
    }

    private List<String> lookupBlurring(String cpr, String clientName) throws URISyntaxException, JAXBException {
        GetBlurredOrganisationsRequest request = new GetBlurredOrganisationsRequest();
        PatientIdType patientId = new PatientIdType();
        BlurredOrganisationsResponse response;

        patientId.setId(cpr);
        patientId.setClassification(PatientIdClassificationType.CPR);

        request.setPatientId(patientId);

        ServiceCaller<GetBlurredOrganisationsRequest, BlurredOrganisationsResponse> serviceCaller =
                new ServiceCaller<>(GetBlurredOrganisationsRequest.class, BlurredOrganisationsResponse.class);
        response = serviceCaller.call(TestIdentities.systembrugerMedClientName(clientName),
                request, SOAPOperation.GetBlurredOrganisations.getSoapOperation(), new URI(idsasLookupUrl));
        return response.getOrganisationId().stream().map(OrganisationIdType::getId).collect(Collectors.toList());
    }

    private void createBlurring(String userType, String cpr, LocalDateTime dateTime) throws DatatypeConfigurationException {
        CreateBlurringRequest request = new CreateBlurringRequest();
        PatientIdType patientId = new PatientIdType();
        Identity identity = TestIdentities.identityFor(userType);

        patientId.setId(cpr);
        patientId.setClassification(PatientIdClassificationType.CPR);
        request.setPatientId(patientId);
        request.setEndDateTime(toXmlGregorian(dateTime));
        try {
            ServiceCaller<CreateBlurringRequest, String> serviceCaller = new ServiceCaller<>(CreateBlurringRequest.class, String.class);
            response = serviceCaller.call(identity, request, SOAPOperation.CreateBlurring.getSoapOperation(), new URI(idsasRegistrationUrl));
        } catch (Exception e) {
            response = e.getMessage();
        }
    }

}