package dk.nsp.idsas.integrations.sores;

import com.fasterxml.jackson.core.JsonProcessingException;
import com.fasterxml.jackson.databind.JsonNode;
import com.fasterxml.jackson.databind.ObjectMapper;
import dk.nsp.idsas.integrations.ServiceErrorCollector;
import org.apache.http.HttpResponse;
import org.apache.http.HttpStatus;
import org.apache.http.client.HttpClient;
import org.apache.http.client.methods.HttpPost;
import org.apache.http.client.methods.HttpUriRequest;
import org.apache.http.entity.StringEntity;
import org.apache.log4j.Logger;

import java.io.IOException;
import java.io.UnsupportedEncodingException;

public class SoresClient {
    private final static Logger logger = Logger.getLogger(SoresClient.class);

    public static String ERROR_MSG = "Fejl i SORES-integrationen";
    public static String ERROR_MSG_BAD_REQUEST = "SORES svarede med status 400";
    public static String ERROR_MSG_INTERNAL_SERVER_ERROR = "SORES svarede med status 500";

    private static final String CONTENT_TYPE = "application/json";

    private final ObjectMapper objectMapper = new ObjectMapper();
    private final String endpoint;
    private final HttpClient httpClient;
    private final ServiceErrorCollector serviceErrorCollector;

    public SoresClient(HttpClient httpClient, String endpoint, ServiceErrorCollector serviceErrorCollector) {
        this.endpoint = endpoint;
        this.httpClient = httpClient;
        this.serviceErrorCollector = serviceErrorCollector;
    }

    public int getErrorCount() {
        return serviceErrorCollector.getErrorCount();
    }

    public String getCvrNumberBySor(String sor) throws SoresClientException {

        // Skal være et gyldigt tal
        if (!sor.matches("\\d+")) {
            serviceErrorCollector.registerServiceError();
            throw new SoresClientException(ERROR_MSG);
        }

        try {
            String body = buildInstitutionOwnerCvrNumberBody(sor);
            HttpUriRequest request = buildSorRequest(body);
            HttpResponse response = invokeSor(request);
            String result = handleResponse(sor, response);
            serviceErrorCollector.registerServiceSuccess();
            return result;
        } catch (UnsupportedOperationException | IOException e) {
            serviceErrorCollector.registerServiceError();
            throw new SoresClientException(ERROR_MSG, e);
        }
    }

    private HttpUriRequest buildSorRequest(String input) throws UnsupportedEncodingException {
        final StringEntity entity = new StringEntity(input);
        final HttpPost httpPost = new HttpPost(endpoint);
        httpPost.setEntity(entity);
        httpPost.setHeader("Content-type", CONTENT_TYPE);
        return httpPost;
    }

    private String buildInstitutionOwnerCvrNumberBody(String inputSor) throws JsonProcessingException {
        final ObjectMapper objectMapper = new ObjectMapper();

        return objectMapper.writeValueAsString(new Object() {
            public final String op = "InstitutionOwnerCvrNumber";
            public final String[] sorIds = new String[]{inputSor};
        });
    }

    private HttpResponse invokeSor(HttpUriRequest request) throws SoresClientException {
        try {
            return httpClient.execute(request);
        } catch (IOException e) {
            serviceErrorCollector.registerServiceError();
            throw new SoresClientException(ERROR_MSG, e);
        }
    }

    private String handleResponse(String inputCode, HttpResponse response)
            throws SoresClientException, IOException {

        if (response.getStatusLine().getStatusCode() == HttpStatus.SC_INTERNAL_SERVER_ERROR) {
            serviceErrorCollector.registerServiceError();
            throw new SoresClientException(ERROR_MSG_INTERNAL_SERVER_ERROR);
        }

        if (response.getStatusLine().getStatusCode() == HttpStatus.SC_BAD_REQUEST) {
            serviceErrorCollector.registerServiceError();
            throw new SoresClientException(ERROR_MSG_BAD_REQUEST);
        }

        JsonNode jsonNode = objectMapper.readTree(response.getEntity().getContent());
        if (jsonNode.size() == 0) {
            logger.warn("SORES returned empty response");
            return null;
        } else {
            return jsonNode.get(inputCode).asText();
        }
    }

}
