package dk.nsp.idsas.integrations.personinformation;

import dk.nsp.idsas.integrations.ServiceErrorCollector;

import org.apache.http.HttpResponse;
import org.apache.http.client.HttpClient;
import org.apache.http.client.methods.HttpUriRequest;
import org.junit.Before;
import org.junit.Test;
import org.mockito.Mockito;

import java.io.IOException;

import static dk.nsp.idsas.integrations.util.TestUtil.createHttpResponse;
import static org.junit.Assert.*;

public class PersonInformationClientTest {

    private HttpClient httpClient;
    private final String url = "test-url";
    private ServiceErrorCollector serviceErrorCollector;
    private PersonInformationClient personInformationClient;

    @Before
    public void setUp() {
        httpClient = Mockito.mock(HttpClient.class);
        serviceErrorCollector = Mockito.mock(ServiceErrorCollector.class);
        personInformationClient = new PersonInformationClient(httpClient, url, serviceErrorCollector);
    }

    @Test
    public void personExists_WhenServiceReturnsValidResponse_ReturnsTrueAndRegistersServiceSuccess() throws PersonInformationClientException, IOException {
        HttpResponse httpResponse = createHttpResponse(200, "application/json", "{\"cpr\": \"1234\"}");
        Mockito.doAnswer(arg -> httpResponse).when(httpClient).execute(Mockito.any(HttpUriRequest.class));
        String cpr = "1234";

        boolean personExists = personInformationClient.personExists(cpr);

        assertTrue(personExists);
        String expectedPath = url + "/person/" + cpr;
        Mockito.verify(httpClient).execute(
                Mockito.argThat(request -> expectedPath.equals(request.getURI().getPath())));
        Mockito.verify(serviceErrorCollector).registerServiceSuccess();
    }

    @Test
    public void personExists_WhenClientThrowsIOException_ThrowsPersonInformationClientExceptionAndRegistersError() throws IOException {
        Mockito.doThrow(IOException.class).when(httpClient).execute(Mockito.any(HttpUriRequest.class));

        PersonInformationClientException e = assertThrows(PersonInformationClientException.class, () -> personInformationClient.personExists("1234"));

        assertEquals("CPR-nummeret kunne ikke valideres, fordi kaldet til PersonInformation-servicen gik galt.", e.getMessage());
        Mockito.verify(serviceErrorCollector).registerServiceError();
    }

    @Test
    public void personExists_WhenServiceReturnsResponse200WithContentTypeText_ThrowsPersonInformationClientExceptionAndRegistersError() throws IOException {
        HttpResponse httpResponse = createHttpResponse(200, "text/plain", "{\"cpr\": \"1234\"}");
        Mockito.doAnswer(arg -> httpResponse).when(httpClient).execute(Mockito.any(HttpUriRequest.class));

        PersonInformationClientException e = assertThrows(PersonInformationClientException.class, () -> personInformationClient.personExists("1234"));

        assertEquals("Fejl i retursvar fra PersonInformation", e.getMessage());
        Mockito.verify(serviceErrorCollector).registerServiceError();
    }

    @Test
    public void personExists_WhenServiceReturnsResponse200WithInvalidJson_ThrowsPersonInformationClientExceptionAndRegistersError() throws IOException {
        HttpResponse httpResponse = createHttpResponse(200, "application/json", "invalid body");
        Mockito.doAnswer(arg -> httpResponse)
                .when(httpClient).execute(Mockito.any(HttpUriRequest.class));

        PersonInformationClientException e = assertThrows(PersonInformationClientException.class, () -> personInformationClient.personExists("1234"));

        assertEquals("Fejl i fortolkning af svar fra PersonInformation", e.getMessage());
        Mockito.verify(serviceErrorCollector).registerServiceError();
    }

    @Test
    public void personExists_WhenServiceReturnsResponse200WithoutCprEntry_ThrowsPersonInformationClientExceptionAndRegistersError() throws IOException {
        HttpResponse httpResponse = createHttpResponse(200, "application/json", "{\"test\": \"1234\"}");
        Mockito.doAnswer(arg -> httpResponse).when(httpClient).execute(Mockito.any(HttpUriRequest.class));

        PersonInformationClientException e = assertThrows(PersonInformationClientException.class, () -> personInformationClient.personExists("1234"));

        assertEquals("Fejl i retursvar fra PersonInformation", e.getMessage());
        Mockito.verify(serviceErrorCollector).registerServiceError();
    }

    @Test
    public void personExists_WhenServiceReturnsResponse404WithContentTypeText_ThrowsPersonInformationClientExceptionAndRegistersError() throws IOException {
        HttpResponse httpResponse = createHttpResponse(404, "text/plain", "{\"cpr\": \"1234\"}");
        Mockito.doAnswer(arg -> httpResponse).when(httpClient).execute(Mockito.any(HttpUriRequest.class));

        PersonInformationClientException e = assertThrows(PersonInformationClientException.class, () -> personInformationClient.personExists("1234"));

        assertEquals("Fejl i retursvar fra PersonInformation", e.getMessage());
        Mockito.verify(serviceErrorCollector).registerServiceError();
    }

    @Test
    public void personExists_WhenServiceReturnsResponse404WithInvalidJson_ThrowsPersonInformationClientExceptionAndRegistersError() throws IOException {
        HttpResponse httpResponse = createHttpResponse(404, "application/json", "invalid body");
        Mockito.doAnswer(arg -> httpResponse).when(httpClient).execute(Mockito.any(HttpUriRequest.class));

        PersonInformationClientException e = assertThrows(PersonInformationClientException.class, () -> personInformationClient.personExists("1234"));

        assertEquals("Fejl i fortolkning af svar fra PersonInformation", e.getMessage());
        Mockito.verify(serviceErrorCollector).registerServiceError();
    }

    @Test
    public void personExists_WhenServiceReturnsResponse404WithValidJson_ReturnsFalseAndRegistersServiceSuccess() throws PersonInformationClientException, IOException {
        HttpResponse httpResponse = createHttpResponse(404, "application/json", "{\"test\": \"test\"}");
        Mockito.doAnswer(arg -> httpResponse).when(httpClient).execute(Mockito.any(HttpUriRequest.class));

        boolean personExists = personInformationClient.personExists("1234");

        assertFalse(personExists);
        Mockito.verify(serviceErrorCollector).registerServiceSuccess();
    }

    @Test
    public void personExists_WhenServiceReturnsResponse400_ThrowsPersonInformationClientExceptionAndRegistersError() throws IOException {
        HttpResponse httpResponse = createHttpResponse(400, "application/json", "{\"test\": \"test\"}");
        Mockito.doAnswer(arg -> httpResponse).when(httpClient).execute(Mockito.any(HttpUriRequest.class));

        PersonInformationClientException e = assertThrows(PersonInformationClientException.class, () -> personInformationClient.personExists("1234"));

        assertEquals("CPR-nummeret kunne ikke valideres, fordi kaldet til PersonInformation-servicen returnerede en uventet statuskode: 400", e.getMessage());
        Mockito.verify(serviceErrorCollector).registerServiceError();
    }

}