package dk.nsp.idsas.integrations.sores;

import dk.nsp.idsas.integrations.ServiceErrorCollector;
import org.apache.http.HttpResponse;
import org.apache.http.client.HttpClient;
import org.apache.http.client.methods.HttpUriRequest;
import org.junit.Before;
import org.junit.Test;
import org.mockito.Mockito;

import java.io.IOException;

import static dk.nsp.idsas.integrations.sores.TestUtil.createHttpResponse;
import static org.junit.Assert.*;

public class SoresClientTest {
    private HttpClient httpClient;
    private final String url = "test-url";

    private ServiceErrorCollector serviceErrorCollector;
    private SoresClient soresClient;

    @Before
    public void setUp() {
        httpClient = Mockito.mock(HttpClient.class);
        serviceErrorCollector = Mockito.mock(ServiceErrorCollector.class);
        soresClient = new SoresClient(httpClient, url, serviceErrorCollector);
    }

    private String generateJSONResponse(String[] sorCode, String[] cvr) {
        String response = "{";
        for (int i = 0; i < sorCode.length; i++) {
            response += "\"" + sorCode[i] + "\": \"" + cvr[i] + "\"";
            if (i < sorCode.length - 1) {
                response += ", ";
            }
        }
        response += "}";
        return response;
    }

    @Test
    public void getCvrNumberBySor_WhenServiceReturnsValidResponse_ReturnsCvrAndRegistersServiceSuccess() throws IOException, SoresClientException {
        // Arrange
        String[] cvr = {"12345678"};
        String[] sorCode = {"1234"};
        HttpResponse httpResponse = createHttpResponse(200, "application/json", generateJSONResponse(sorCode, cvr));
        Mockito.doAnswer(arg -> httpResponse).when(httpClient).execute(Mockito.any(HttpUriRequest.class));

        // Act
        String returnCvr = soresClient.getCvrNumberBySor(sorCode[0]);

        // Assert
        assertTrue(returnCvr.equals(cvr[0]));
        String expectedPath = url;
        Mockito.verify(httpClient).execute(
                Mockito.argThat(request -> expectedPath.equals(request.getURI().getPath())));
        Mockito.verify(serviceErrorCollector).registerServiceSuccess();
    }

    @Test
    public void getCvrNumberBySor_WhenClientThrowsIOException_ThrowsSoresClientExceptionAndRegisterError() throws IOException {
        // Arrange
        Mockito.doThrow(IOException.class).when(httpClient).execute(Mockito.any(HttpUriRequest.class));

        // Act
        SoresClientException e = assertThrows(SoresClientException.class, () -> soresClient.getCvrNumberBySor("1234"));

        // Assert
        assertTrue(e.getMessage().contains(SoresClient.ERROR_MSG));
    }

    @Test
    public void getCvrNumberBySor_WhenServiceReturnsResponse200WithInvalidJson_ThrowsSoresClientExceptionAndRegistersError() throws IOException {
        // Arrange
        HttpResponse httpResponse = createHttpResponse(200, "application/json", "invalid body");
        Mockito.doAnswer(arg -> httpResponse).when(httpClient).execute(Mockito.any(HttpUriRequest.class));

        // Act
        SoresClientException e = assertThrows(SoresClientException.class, () -> soresClient.getCvrNumberBySor("1234"));

        // Assert
        assertTrue(e.getMessage().contains(SoresClient.ERROR_MSG));
    }

    @Test
    public void getCvrNumberBySor_WhenServiceReturnsResponse404WithInvalidJson_ThrowsSoresClientExceptionAndRegistersError() throws IOException {
        // Arrange
        HttpResponse httpResponse = createHttpResponse(404, "application/json", "invalid body");
        Mockito.doAnswer(arg -> httpResponse).when(httpClient).execute(Mockito.any(HttpUriRequest.class));

        // Act
        SoresClientException e = assertThrows(SoresClientException.class, () -> soresClient.getCvrNumberBySor("1234"));

        // Assert
        assertTrue(e.getMessage().contains(SoresClient.ERROR_MSG));
    }

    @Test
    public void getCvrNumberBySor_WhenServiceReturnsResponse400_ThrowsSoresClientExceptionAndRegistersError() throws IOException {
        // Arrange
        HttpResponse httpResponse = createHttpResponse(400, "application/json", "invalid body");
        Mockito.doAnswer(arg -> httpResponse).when(httpClient).execute(Mockito.any(HttpUriRequest.class));

        // Act
        SoresClientException e = assertThrows(SoresClientException.class, () -> soresClient.getCvrNumberBySor("1234"));

        // Assert
        assertTrue(e.getMessage().contains(SoresClient.ERROR_MSG_BAD_REQUEST));
    }

    @Test
    public void getCvrNumberBySor_WhenServiceReturnsResponse500_ThrowsSoresClientExceptionAndRegistersError() throws IOException {
        // Arrange
        HttpResponse httpResponse = createHttpResponse(500, "application/json", "invalid body");
        Mockito.doAnswer(arg -> httpResponse).when(httpClient).execute(Mockito.any(HttpUriRequest.class));

        // Act
        SoresClientException e = assertThrows(SoresClientException.class, () -> soresClient.getCvrNumberBySor("1234"));

        // Assert
        assertTrue(e.getMessage().contains(SoresClient.ERROR_MSG_INTERNAL_SERVER_ERROR));
    }

    @Test
    public void getCvrNumberBySor_WhenSorCodeIsNotInteger_IgnoreSorCode() throws IOException, SoresClientException {
        // Arrange
        String[] cvr = {"12345678"};
        String[] sorCode = {"1234", "ABC"};
        String[] sorCodeValidated = {"1234"};
        HttpResponse httpResponse = createHttpResponse(200, "application/json", generateJSONResponse(sorCodeValidated, cvr));
        Mockito.doAnswer(arg -> httpResponse).when(httpClient).execute(Mockito.any(HttpUriRequest.class));

        // Act
        String returnCvr = soresClient.getCvrNumberBySor(sorCode[0]);

        // Assert
        assertTrue(returnCvr.equals(cvr[0]));
        String expectedPath = url;
        Mockito.verify(httpClient).execute(
                Mockito.argThat(request -> expectedPath.equals(request.getURI().getPath())));
        Mockito.verify(serviceErrorCollector).registerServiceSuccess();
    }
}