package dk.nsp.idsas.ws._20240124;

import com.sun.xml.ws.developer.SchemaValidation;
import dk.nsp.idsas.dto.OrganizationIdDto;
import dk.nsp.idsas.dto.PersonIdType;
import dk.nsp.idsas.security.IdsasActor;
import dk.nsp.idsas.security.IdsasActorBuilder;
import dk.nsp.idsas.service.IdsasServiceException;
import dk.nsp.idsas.service.LookupService;
import dk.nsp.idsas.service.OrgBlurringService;
import dk.nsp.idsas.util.FaultCode;
import dk.nsp.idsas.util.IdsasException;
import dk.nsp.idsas.util.SoapFaultGenerator;
import dk.nsp.idsas.ws.IdsasAbstractWS;
import dk.sundhedsdatastyrelsen.identitetssloering._2024._01._24.*;
import org.apache.log4j.Logger;
import org.springframework.beans.factory.annotation.Autowired;

import javax.jws.WebService;
import javax.servlet.annotation.WebServlet;
import javax.xml.ws.BindingType;
import java.util.List;
import java.util.stream.Collectors;

import static javax.xml.ws.soap.SOAPBinding.SOAP11HTTP_BINDING;

@WebServlet(name = "IdsasLookupService2024", urlPatterns = {"/20240124/lookup"})
@WebService(serviceName = "IdsasLookupService2024", wsdlLocation = "wsdl/20240124/idsas-lookup.wsdl",
        targetNamespace = "http://sundhedsdatastyrelsen.dk/identitetssloering/2024/01/24/",
        portName = "IdsasLookupServicePort")
@BindingType(value = SOAP11HTTP_BINDING)
@SchemaValidation
public class IdsasLookupServiceWS extends IdsasAbstractWS implements IdsasLookupServicePortType {
    private static final Logger logger = Logger.getLogger(IdsasLookupServiceWS.class);

    @Autowired
    private IdsasActorBuilder idsasActorBuilder;

    @Autowired
    private LookupService lookupService;

    @Autowired
    private OrgBlurringService orgBlurringService;

    @Override
    public BlurredOrganisationsResponse getBlurredOrganisations(GetBlurredOrganisationsRequest getBlurredOrganisationsRequest) {
        String patientCpr = getBlurredOrganisationsRequest.getPatientId().getId();
        logger.trace(String.format("Enter get blurred organisations for patient %s", patientCpr));
        PatientIdClassificationType patientIdType = getBlurredOrganisationsRequest.getPatientId().getClassification();

        // Liste af blurrings hentes
        BlurredOrganisationsResponse entityList = new BlurredOrganisationsResponse();
        try {
            IdsasActor actor = idsasActorBuilder.buildActor();
            List<OrganizationIdDto> orgs = lookupService.getActiveBlurrings(actor, patientCpr, mapPersonIdType(patientIdType));
            entityList.getOrganisationId().addAll(orgs.stream().map(IdsasLookupServiceWS::mapToOrganisationId).collect(Collectors.toList()));
            return entityList;

        } catch (IdsasServiceException e) {
            logger.error(e.getMessage(), e);
            throw SoapFaultGenerator.generateSOAPFault(e.isClient(), e.getMessage(), e.getCode().getValue());
        } catch (Exception e) {
            logger.error("Failed to get blurred organisations", e);
            throw SoapFaultGenerator.generateSOAPFault(false, "Internal server error",
                    FaultCode.LOOKUP_ERROR.getValue());
        }
    }

    @Override
    public ListOrgBlurringsForCVRResponse listOrgBlurringsForCVR(ListOrgBlurringsForCVRRequest emptyInput) {
        IdsasActor actor;
        try {
            actor = idsasActorBuilder.buildActor();
        } catch (IdsasException e) {
            logger.error(e.getMessage(), e);
            throw SoapFaultGenerator.generateSOAPFault(e.isClient(), e.getMessage(), e.getCode().getValue());
        }

        try {
            List<OrganizationIdDto> orgs = orgBlurringService.listOrgBlurringsForCVR(actor.getOrganizationId(), actor.getActorType());
            List<OrganisationIdType> orgIdTypes = orgs.stream()
                    .map(o -> {
                        OrganisationIdType orgIdType = new OrganisationIdType();
                        OrganisationIdClassificationType classification = OrganisationIdClassificationType.fromValue(o.idType.name());
                        orgIdType.setId(o.id);
                        orgIdType.setClassification(classification);
                        return orgIdType;
                    })
                    .collect(Collectors.toList());

            ListOrgBlurringsForCVRResponse response = new ListOrgBlurringsForCVRResponse();
            response.getOrg().addAll(orgIdTypes);
            return response;

        } catch (IdsasServiceException e) {
            logger.error(e.getMessage(), e);
            throw SoapFaultGenerator.generateSOAPFault(e.isClient(), e.getMessage(), e.getCode().getValue());
        }
    }

    @Override
    public ListAllActiveOrgBlurringsResponse listAllActiveOrgBlurrings(ListAllActiveOrgBlurringsRequest emptyInput) {
        IdsasActor actor;
        try {
            actor = idsasActorBuilder.buildActor();
        } catch (IdsasException e) {
            logger.error(e.getMessage(), e);
            throw SoapFaultGenerator.generateSOAPFault(e.isClient(), e.getMessage(), e.getCode().getValue());
        }

        try {
            List<OrganizationIdDto> orgs = orgBlurringService.listAllActiveOrgBlurrings(actor.getClientSystemName(), actor.getActorType());
            List<OrganisationIdType> orgIdTypes = orgs.stream()
                    .map(o -> {
                        OrganisationIdType orgIdType = new OrganisationIdType();
                        OrganisationIdClassificationType classification = OrganisationIdClassificationType.fromValue(o.idType.name());
                        orgIdType.setId(o.id);
                        orgIdType.setClassification(classification);
                        return orgIdType;
                    })
                    .collect(Collectors.toList());

            ListAllActiveOrgBlurringsResponse response = new ListAllActiveOrgBlurringsResponse();
            response.getOrg().addAll(orgIdTypes);
            return response;

        } catch (IdsasServiceException e) {
            logger.error(e.getMessage(), e);
            throw SoapFaultGenerator.generateSOAPFault(e.isClient(), e.getMessage(), e.getCode().getValue());
        }
    }

    private static OrganisationIdType mapToOrganisationId(OrganizationIdDto dto) {
        OrganisationIdType orgId = new OrganisationIdType();
        orgId.setId(dto.id);
        orgId.setClassification(OrganisationIdClassificationType.fromValue(dto.idType.name()));
        return orgId;
    }

    private static PersonIdType mapPersonIdType(PatientIdClassificationType classification) {
        return PersonIdType.valueOf(classification.value());
    }
}
