package dk.nsp.idsas.setup;

import dk.nsp.idsas.dao.BlurringDao;
import dk.nsp.idsas.dao.OrgBlurringDao;
import dk.nsp.idsas.dao.WhitelistDao;
import dk.nsp.idsas.http.PoolingHttpClientFactory;
import dk.nsp.idsas.integrations.ServiceErrorCollector;
import dk.nsp.idsas.integrations.personinformation.PersonInformationClient;
import dk.nsp.idsas.integrations.sores.SoresClient;
import dk.nsp.idsas.service.BlurringService;
import dk.nsp.idsas.service.OrgBlurringService;
import dk.nsp.idsas.util.AuditLogger;
import dk.nsp.idsas.util.health.PersonInformationHealthIndicator;
import dk.nsp.idsas.util.health.SoresClientHealthIndicator;
import org.apache.http.client.HttpClient;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.context.annotation.Bean;
import org.springframework.context.annotation.Configuration;
import org.springframework.context.annotation.PropertySource;
import org.springframework.context.annotation.PropertySources;

import java.time.Duration;
import java.time.Period;

@Configuration
@PropertySources({
        @PropertySource("classpath:/META-INF/maven/dk.nsp.idsas/idsas-registration-web/pom.properties"),
        @PropertySource(value = "idsas-registration.properties", encoding = "ISO-8859-1"),
        @PropertySource(value = "idsas-patient-id-salt.properties", encoding = "ISO-8859-1")
})
public class WebSetup {
    @Value("${personinformation.url}")
    private String personInformationUrl;

    @Value("${personinformation.errorcount.duration:PT10M}")
    private String personInformationErrorCountDuration;

    @Value("${personinformation.error.tolerance:0}")
    private int personInformationErrorTolerance;

    @Value("${soresclient.url}")
    private String soresClientUrl;

    @Value("${soresclient.errorcount.duration:PT10M}")
    private String soresClientErrorCountDuration;

    @Value("${soresclient.error.tolerance:0}")
    private int soresClientErrorTolerance;

    @Value("${idsas.patient.id.saltPrefix:#{null}}")
    private String patientIdSaltPrefix;

    @Value("${idsas.patient.id.saltPostfix:#{null}}")
    private String patientIdSaltPostfix;

    @Value("${idsas.httpclient.pooling.totalconnections:200}")
    private int maxTotalConnections;

    @Value("${idsas.httpclient.pooling.maxconnections.pr.route:20}")
    private int defaultMaxConnectionsPerRoute;

    @Value("${idsas.blurring.expiry.max.days:P30D}")
    private String expiryMaxDays;

    @Value("${idsas.allowed.client.system.names}")
    private String[] allowedClientSystemNames;

    @Bean
    public PoolingHttpClientFactory poolingHttpClientFactory() {
        return new PoolingHttpClientFactory(maxTotalConnections, defaultMaxConnectionsPerRoute);
    }

    @Bean
    public PersonInformationClient personInformationClient(PoolingHttpClientFactory poolingHttpClientFactory) {
        HttpClient httpClient = poolingHttpClientFactory.getClient();
        return new PersonInformationClient(httpClient, personInformationUrl, Duration.parse(personInformationErrorCountDuration));
    }

    @Bean
    public SoresClient soresClient(PoolingHttpClientFactory poolingHttpClientFactory) {
        HttpClient httpClient = poolingHttpClientFactory.getClient();
        ServiceErrorCollector serviceErrorCollector = new ServiceErrorCollector(Duration.parse(soresClientErrorCountDuration));
        return new SoresClient(httpClient, soresClientUrl, serviceErrorCollector);
    }

    @Bean
    public BlurringService blurringService(WhitelistDao whitelistDao, BlurringDao blurringDao,
                                           AuditLogger auditLogger, PersonInformationClient personInformationClient) {
        // If patientIdSaltEncoded is null or empty
        if (patientIdSaltPrefix == null || patientIdSaltPrefix.isEmpty() || patientIdSaltPostfix == null || patientIdSaltPostfix.isEmpty()) {
            throw new IllegalStateException("patientIdSalt cannot be empty or null");
        }

        return new BlurringService(whitelistDao, blurringDao, patientIdSaltPrefix, patientIdSaltPostfix, auditLogger, personInformationClient,
                Period.parse(expiryMaxDays).getDays());
    }

    @Bean
    public OrgBlurringService orgBlurringService(WhitelistDao whitelistDao, OrgBlurringDao orgBlurringDao, SoresClient soresClient) {
        return new OrgBlurringService(whitelistDao, orgBlurringDao, soresClient, allowedClientSystemNames);
    }

    @Bean
    public PersonInformationHealthIndicator personInformationHealthIndicator(PersonInformationClient personInformationClient) {
        return new PersonInformationHealthIndicator(personInformationClient, personInformationErrorTolerance);
    }

    @Bean
    public SoresClientHealthIndicator soresClientHealthIndicator(SoresClient soresClient) {
        return new SoresClientHealthIndicator(soresClient, soresClientErrorTolerance);
    }

}
