package dk.nsp.idsas.ws._20240124;

import dk.nsp.idsas.security.IdsasActor;
import dk.nsp.idsas.security.IdsasActorBuilder;
import dk.nsp.idsas.security.IdsasSecurityException;
import dk.nsp.idsas.service.BlurringService;
import dk.nsp.idsas.service.IdsasServiceException;
import dk.nsp.idsas.service.OrgBlurringService;
import dk.nsp.idsas.util.FaultCode;
import dk.sundhedsdatastyrelsen.identitetssloering._2024._01._24.*;
import org.junit.Before;
import org.junit.Test;
import org.mockito.InjectMocks;
import org.mockito.Mock;
import org.mockito.MockitoAnnotations;

import javax.xml.datatype.DatatypeConfigurationException;
import javax.xml.datatype.DatatypeFactory;
import javax.xml.datatype.XMLGregorianCalendar;
import javax.xml.ws.soap.SOAPFaultException;
import java.time.ZonedDateTime;
import java.util.GregorianCalendar;

import static com.sun.tools.ws.wsdl.parser.Util.fail;
import static org.junit.Assert.assertEquals;
import static org.mockito.Mockito.*;

public class IdsasRegistrationWSTest {

    @Mock
    private IdsasActorBuilder idsasActorBuilder;

    @Mock
    private BlurringService blurringService;

    @Mock
    private OrgBlurringService orgBlurringService;

    @Mock
    private CreateBlurringRequest createBlurringRequest;

    @Mock
    private PatientIdType patientIdType;

    @InjectMocks
    private IdsasRegistrationWS idsasRegistrationWS;

    @Before
    public void setUp() throws IdsasSecurityException {
        MockitoAnnotations.openMocks(this);
        when(createBlurringRequest.getPatientId()).thenReturn(patientIdType);
        when(patientIdType.getId()).thenReturn("1234567890");
        when(patientIdType.getClassification()).thenReturn(PatientIdClassificationType.CPR);
        when(idsasActorBuilder.buildActor()).thenReturn(mock(IdsasActor.class));
    }

    @Test
    public void testCreateBlurring() throws IdsasSecurityException, DatatypeConfigurationException, IdsasServiceException {
        // Arrange
        IdsasActor actor = mock(IdsasActor.class);
        when(idsasActorBuilder.buildActor()).thenReturn(actor);

        // Mock endDateTime
        GregorianCalendar gregorianCalendar = new GregorianCalendar();
        gregorianCalendar.setTimeInMillis(ZonedDateTime.now().toInstant().toEpochMilli());
        XMLGregorianCalendar xmlGregorianCalendar = DatatypeFactory.newInstance().newXMLGregorianCalendar(gregorianCalendar);
        when(createBlurringRequest.getEndDateTime()).thenReturn(xmlGregorianCalendar);

        // Act
        idsasRegistrationWS.createBlurring(createBlurringRequest);

        // Assert
        verify(blurringService, times(1)).createBlurring(
                any(), any(), any(), any(), any(ZonedDateTime.class));
    }

    @Test
    public void testCreateBlurring_IdsasServiceException() throws IdsasSecurityException, IdsasServiceException, DatatypeConfigurationException {
        // Arrange
        IdsasActor actor = mock(IdsasActor.class);
        when(idsasActorBuilder.buildActor()).thenReturn(actor);
        IdsasServiceException idsasServiceException = new IdsasServiceException("test", FaultCode.CREATE_BLURRING_ERROR, true);
        doThrow(idsasServiceException).when(blurringService).createBlurring(
                any(), any(), any(), any(), any(ZonedDateTime.class));

        // Mock endDateTime
        GregorianCalendar gregorianCalendar = new GregorianCalendar();
        gregorianCalendar.setTimeInMillis(ZonedDateTime.now().toInstant().toEpochMilli());
        XMLGregorianCalendar xmlGregorianCalendar = DatatypeFactory.newInstance().newXMLGregorianCalendar(gregorianCalendar);
        when(createBlurringRequest.getEndDateTime()).thenReturn(xmlGregorianCalendar);

        try {
            // Act
            idsasRegistrationWS.createBlurring(createBlurringRequest);
            fail("Expected an exception to be thrown");
        } catch (SOAPFaultException e) {

            // Assert
            assertEquals("test", e.getMessage());
        }
    }

    @Test
    public void testCreateBlurring_RuntimeException() throws IdsasSecurityException, IdsasServiceException, DatatypeConfigurationException {
        // Arrange
        IdsasActor actor = mock(IdsasActor.class);
        when(idsasActorBuilder.buildActor()).thenReturn(actor);
        doThrow(new RuntimeException()).when(blurringService).createBlurring(
                any(), any(), any(), any(), any(ZonedDateTime.class));

        // Mock endDateTime
        GregorianCalendar gregorianCalendar = new GregorianCalendar();
        gregorianCalendar.setTimeInMillis(ZonedDateTime.now().toInstant().toEpochMilli());
        XMLGregorianCalendar xmlGregorianCalendar = DatatypeFactory.newInstance().newXMLGregorianCalendar(gregorianCalendar);
        when(createBlurringRequest.getEndDateTime()).thenReturn(xmlGregorianCalendar);

        try {
            // Act
            idsasRegistrationWS.createBlurring(createBlurringRequest);
            fail("Expected an exception to be thrown");
        } catch (SOAPFaultException e) {

            // Assert
            assertEquals("Internal server error", e.getMessage());
        }
    }

    @Test
    public void createOrgBlurring() throws IdsasSecurityException, IdsasServiceException {
        // Arrange
        IdsasActor actor = mock(IdsasActor.class);
        when(actor.getActorType()).thenReturn(IdsasActor.IdsasActorType.SYSTEM);
        when(actor.getOrganizationIdType()).thenReturn("CVR");

        when(idsasActorBuilder.buildActor()).thenReturn(actor);
        CreateOrgBlurringRequest createOrgBlurringRequestType = mock(CreateOrgBlurringRequest.class);
        when(createOrgBlurringRequestType.getOrg()).thenReturn(
                mock(OrganisationIdType.class));
        when(createOrgBlurringRequestType.getOrg().getId()).thenReturn("1234567890");
        when(createOrgBlurringRequestType.getOrg().getClassification()).thenReturn(OrganisationIdClassificationType.SOR);

        // Act
        idsasRegistrationWS.createOrgBlurring(createOrgBlurringRequestType);

        // Assert
        verify(orgBlurringService, times(1)).createOrgBlurring(any(), any(), any(), any());
    }

    @Test
    public void createOrgBlurring_IdsasServiceException() throws IdsasSecurityException, IdsasServiceException {
        // Arrange
        IdsasActor actor = mock(IdsasActor.class);
        when(actor.getActorType()).thenReturn(IdsasActor.IdsasActorType.SYSTEM);
        when(actor.getOrganizationIdType()).thenReturn("CVR");

        when(idsasActorBuilder.buildActor()).thenReturn(actor);
        CreateOrgBlurringRequest createOrgBlurringRequestType = mock(CreateOrgBlurringRequest.class);
        when(createOrgBlurringRequestType.getOrg()).thenReturn(
                mock(OrganisationIdType.class));
        when(createOrgBlurringRequestType.getOrg().getId()).thenReturn("1234567890");
        when(createOrgBlurringRequestType.getOrg().getClassification()).thenReturn(OrganisationIdClassificationType.SOR);

        IdsasServiceException idsasServiceException = new IdsasServiceException("test", FaultCode.CREATE_ORG_BLURRING_ERROR, true);
        doThrow(idsasServiceException).when(orgBlurringService).createOrgBlurring(any(), any(), any(), any());

        try {
            // Act
            idsasRegistrationWS.createOrgBlurring(createOrgBlurringRequestType);
            fail("Expected an exception to be thrown");
        } catch (SOAPFaultException e) {

            // Assert
            assertEquals("test", e.getMessage());
        }
    }

    @Test
    public void removeOrgBlurring() throws IdsasSecurityException, IdsasServiceException {
        // Arrange
        IdsasActor actor = mock(IdsasActor.class);
        when(actor.getActorType()).thenReturn(IdsasActor.IdsasActorType.SYSTEM);
        when(actor.getOrganizationIdType()).thenReturn("CVR");
        when(idsasActorBuilder.buildActor()).thenReturn(actor);
        RemoveOrgBlurringRequest blurredOrgIdType = mock(RemoveOrgBlurringRequest.class);
        when(blurredOrgIdType.getOrg()).thenReturn(mock(OrganisationIdType.class));
        when(blurredOrgIdType.getOrg().getId()).thenReturn("123");
        when(blurredOrgIdType.getOrg().getClassification()).thenReturn(OrganisationIdClassificationType.SOR);

        // Act
        String response = idsasRegistrationWS.removeOrgBlurring(blurredOrgIdType);

        // Assert
        assertEquals("OK", response);
        verify(orgBlurringService, times(1)).removeOrgBlurring(any(), any(), any());
    }

}