package dk.nsp.idsas.service;

import dk.nsp.idsas.dao.BlurringDao;
import dk.nsp.idsas.dao.entity.BlurringEntity;
import dk.nsp.idsas.dto.OrgIdType;
import dk.nsp.idsas.dto.OrganizationIdDto;
import dk.nsp.idsas.dto.PersonIdType;
import dk.nsp.idsas.security.IdsasActor;
import dk.nsp.idsas.util.AuditLogger;
import dk.nsp.idsas.util.ErrorMessages;
import dk.nsp.idsas.util.FaultCode;
import dk.nsp.idsas.util.HashingUtil;
import org.springframework.dao.DataAccessException;

import java.util.Arrays;
import java.util.List;
import java.util.stream.Collectors;

public class LookupService {
    private final BlurringDao blurringDao;
    private final AuditLogger auditLogger;
    private final String[] allowedClientSystemNames;
    private final String patientIdSaltPrefix;
    private final String patientIdSaltPostfix;

    public LookupService(BlurringDao blurringDao, String patientIdSaltPrefix, String patientIdSaltPostfix,
                         AuditLogger auditLogger, String[] allowedClientSystemNames) {
        this.blurringDao = blurringDao;
        this.auditLogger = auditLogger;
        this.allowedClientSystemNames = allowedClientSystemNames;
        this.patientIdSaltPrefix = patientIdSaltPrefix;
        this.patientIdSaltPostfix = patientIdSaltPostfix;
    }
    
    public List<OrganizationIdDto> getActiveBlurrings(IdsasActor actor, String patientId, PersonIdType patientIdType) throws IdsasServiceException {
        // Validering af, at actor er en systembruger
        if (!actor.getActorType().equals(IdsasActor.IdsasActorType.SYSTEM)) {
            throw new IdsasServiceException(ErrorMessages.userTypeNotSupported(actor.getActorType().name()), FaultCode.LOOKUP_ERROR, true);
        }

        // Validering af, at systembrugeren har et clientName med den korrekte værdi
        if (!Arrays.asList(allowedClientSystemNames).contains(actor.getClientSystemName())) {
            String names = String.join(", ", allowedClientSystemNames);
            throw new IdsasServiceException(ErrorMessages.clientSystemNameError(
                    actor.getClientSystemName(), names), FaultCode.LOOKUP_ERROR, true);
        }

        // Hente entiteter
        String hashedPatientId = HashingUtil.createPatientIdHash(patientId, patientIdSaltPrefix, patientIdSaltPostfix);

        List<BlurringEntity> entityList;
        try {
            entityList = blurringDao.getActiveBlurrings(hashedPatientId, patientIdType.name());
        } catch (DataAccessException e) {
            throw new IdsasServiceException(ErrorMessages.daoAccessError(), FaultCode.LOOKUP_ERROR, false, e);
        }

        // Mappe entiteter til liste af id'ere
        List<OrganizationIdDto> orgIdList = entityList.stream().map(
            e -> new OrganizationIdDto(e.getOrganisationId(), OrgIdType.valueOf(e.getOrganisationIdType()))
        ).collect(Collectors.toList());

        // Audit logning
        auditLogger.logGetBlurredOrganisations(patientId, patientIdType.name());

        return orgIdList;
    }
}

