package dk.nsp.idsas.service;

import dk.nsp.idsas.dao.SaltDao;
import dk.nsp.idsas.security.IdsasActor;
import dk.nsp.idsas.util.AuditLogger;
import dk.nsp.idsas.util.ErrorMessages;
import dk.nsp.idsas.util.FaultCode;
import org.springframework.dao.DataAccessException;

import java.security.SecureRandom;
import java.util.Arrays;

public class SaltService {
    private final SaltDao saltDao;
    private final AuditLogger auditLogger;
    private final String[] allowedClientSystemNames;
    private final SecureRandom secureRandom;

    public SaltService(SaltDao saltDao, AuditLogger auditLogger, SecureRandom secureRandom, String[] allowedClientSystemNames) {
        this.saltDao = saltDao;
        this.auditLogger = auditLogger;
        this.secureRandom = secureRandom;
        this.allowedClientSystemNames = allowedClientSystemNames;
    }

    public byte[] getCurrentSalt(IdsasActor actor) throws IdsasServiceException {
        validateSystemUser(actor.getActorType());
        validateClientAllowed(actor.getClientSystemName());

        byte[] salt;
        try {
            salt = saltDao.getLatestSalt();
        } catch (DataAccessException e) {
            throw new IdsasServiceException(ErrorMessages.daoAccessError(), FaultCode.GET_SALT_ERROR, false, e);
        }

        auditLogger.logGetCurrentSalt(actor.getClientKey(), "cvr", salt);
        return salt;
    }

    public void renewSalt() {
        byte[] salt = new byte[16];
        secureRandom.nextBytes(salt);
        saltDao.insert(salt);
    }

    private void validateSystemUser(IdsasActor.IdsasActorType actorType) throws IdsasServiceException {
        if (!actorType.equals(IdsasActor.IdsasActorType.SYSTEM)) {
            throw new IdsasServiceException(ErrorMessages.userTypeNotSupported(actorType.name()),
                    FaultCode.GET_SALT_ERROR, true);
        }
    }

    private void validateClientAllowed(String clientSystemName) throws IdsasServiceException {
        if (!Arrays.asList(allowedClientSystemNames).contains(clientSystemName)) {
            throw new IdsasServiceException(ErrorMessages.clientSystemNameError(
                    clientSystemName,
                    Arrays.stream(allowedClientSystemNames).reduce((a, b) -> a + ", " + b).orElse("")
            ), FaultCode.GET_SALT_ERROR, true);
        }
    }
}
