package dk.nsp.idsas.service;

import dk.nsp.idsas.dao.BlurringDao;
import dk.nsp.idsas.dao.WhitelistDao;
import dk.nsp.idsas.dto.OrgIdType;
import dk.nsp.idsas.dto.PersonIdType;
import dk.nsp.idsas.integrations.personinformation.PersonInformationClient;
import dk.nsp.idsas.integrations.personinformation.PersonInformationClientException;
import dk.nsp.idsas.security.IdsasActor;
import dk.nsp.idsas.util.AuditLogger;
import dk.nsp.idsas.util.ErrorMessages;
import dk.nsp.idsas.util.HashingUtil;
import dk.nsp.idsas.util.WhitelistType;
import org.junit.Before;
import org.junit.Test;
import org.mockito.Mockito;

import java.time.ZonedDateTime;

import static org.junit.Assert.assertEquals;
import static org.junit.Assert.assertThrows;

public class BlurringServiceTest {
    private WhitelistDao whitelistDao;
    private BlurringDao blurringDao;
    private AuditLogger auditLogger;
    private PersonInformationClient personInformationClient;
    private BlurringService blurringService;
    private final String patientIdSaltPrefix = "123";
    private final String patientIdSaltPostfix = "456";
    private int maxExpiryDays = 90;

    @Before
    public void setUp() {
        whitelistDao = Mockito.mock(WhitelistDao.class);
        blurringDao = Mockito.mock(BlurringDao.class);
        auditLogger = Mockito.mock(AuditLogger.class);
        personInformationClient = Mockito.mock(PersonInformationClient.class);
        blurringService = new BlurringService(whitelistDao, blurringDao, patientIdSaltPrefix, patientIdSaltPostfix, auditLogger, personInformationClient, maxExpiryDays);
    }

    @Test
    public void createBlurring_WhenCvrIsNotWhitelisted_ThrowsException() {
        // Arrange
        IdsasActor actor = IdsasActor.createSystem("5678", "CVR", "TestSystem", "");
        Mockito.when(whitelistDao.isWhitelisted(actor.getOrganizationId(), WhitelistType.BLURRING)).thenReturn(false);

        // Act
        IdsasServiceException e = assertThrows(IdsasServiceException.class,
                () -> blurringService.createBlurring(actor.getOrganizationId(), OrgIdType.CVR, "", PersonIdType.CPR, ZonedDateTime.now()));

        // Assert
        assertEquals(ErrorMessages.cvrWhitelistError(actor.getOrganizationId(), WhitelistType.BLURRING), e.getMessage());
    }

    @Test
    public void createBlurring_WhenNoPersonInformationIsFoundForCpr_ThrowsException() throws PersonInformationClientException {
        // Arrange
        IdsasActor actor = IdsasActor.createSystem("5678", "CVR", "TestSystem", "");
        String cpr = "1111111111";
        Mockito.when(whitelistDao.isWhitelisted(actor.getOrganizationId(), WhitelistType.BLURRING)).thenReturn(true);
        Mockito.when(personInformationClient.personExists(cpr)).thenReturn(false);

        // Act
        IdsasServiceException e = assertThrows(IdsasServiceException.class,
                () -> blurringService.createBlurring(actor.getOrganizationId(), OrgIdType.CVR, cpr, PersonIdType.CPR, ZonedDateTime.now()));

        // Assert
        assertEquals(ErrorMessages.cprIsNotValidError(cpr), e.getMessage());
    }

    @Test
    public void createBlurring_WhenAllValidationsSucceed_CreatesBlurringAndAuditLogs() throws PersonInformationClientException, IdsasServiceException {
        // Arrange
        IdsasActor actor = IdsasActor.createSystem("5678", "CVR", "TestSystem", "");
        String patientIdType = "CPR";
        String patientId = "1111111111";
        String hashedPatientId = HashingUtil.createPatientIdHash(patientId, "123", "456");
        ZonedDateTime endDateTime = ZonedDateTime.now();
        Mockito.when(whitelistDao.isWhitelisted(actor.getOrganizationId(), WhitelistType.BLURRING)).thenReturn(true);
        Mockito.when(personInformationClient.personExists(patientId)).thenReturn(true);
        // Act
        blurringService.createBlurring(actor.getOrganizationId(), OrgIdType.CVR, patientId, PersonIdType.CPR, endDateTime);

        // Assert
        Mockito.verify(blurringDao).insert(Mockito.argThat(blurring -> {
            assertEquals(hashedPatientId, blurring.getPatientIdHash());
            assertEquals(patientIdType, blurring.getPatientIdType());
            assertEquals(actor.getOrganizationId(), blurring.getOrganisationId());
            assertEquals(OrgIdType.CVR.name(), blurring.getOrganisationIdType());
            assertEquals(endDateTime.toInstant(), blurring.getExpiryDate());
            return true;
        }));
        Mockito.verify(auditLogger).logCreateBlurring(patientId, "CPR", hashedPatientId, actor.getOrganizationId(), OrgIdType.CVR.name(), endDateTime.toInstant());
    }

    @Test
    public void createBlurring_WhenExpiryDateIsExceeded_ThrowsException() throws PersonInformationClientException {
        // Arrange
        IdsasActor actor = IdsasActor.createSystem("5678", "CVR", "TestSystem", "");
        ZonedDateTime exceededDate = ZonedDateTime.now().plusYears(2).plusDays(1);
        String patientId = "1111111111";
        Mockito.when(whitelistDao.isWhitelisted(actor.getOrganizationId(), WhitelistType.BLURRING)).thenReturn(true);
        Mockito.when(personInformationClient.personExists(patientId)).thenReturn(true);

        // Act
        IdsasServiceException e = assertThrows(IdsasServiceException.class,
                () -> blurringService.createBlurring(actor.getOrganizationId(), OrgIdType.CVR, patientId, PersonIdType.CPR, exceededDate));

        // Assert
        assertEquals(ErrorMessages.maxExpiryDateExceeded(maxExpiryDays), e.getMessage());
    }

}
