package dk.nsp.idsas.service;

import dk.nsp.idsas.dao.BlurringDao;
import dk.nsp.idsas.dao.entity.BlurringEntity;
import dk.nsp.idsas.dto.OrgIdType;
import dk.nsp.idsas.dto.OrganizationIdDto;
import dk.nsp.idsas.dto.PersonIdType;
import dk.nsp.idsas.security.IdsasActor;
import dk.nsp.idsas.util.AuditLogger;
import dk.nsp.idsas.util.ErrorMessages;
import dk.nsp.idsas.util.HashingUtil;
import org.junit.Before;
import org.junit.Test;
import org.mockito.Mockito;

import java.util.Arrays;
import java.util.Collections;
import java.util.List;

import static org.junit.Assert.*;

public class LookupServiceTest {

    private final String[] allowedClientSystemNames = {"TestSystem", "TestSystem2"};
    private final String patientIdSaltPrefix = "123";
    private final String patientIdSaltPostfix = "456";

    private LookupService lookupService;
    private BlurringDao blurringDao;
    private AuditLogger auditLogger;

    @Before
    public void setUp() {
        blurringDao = Mockito.mock(BlurringDao.class);
        auditLogger = Mockito.mock(AuditLogger.class);
        lookupService = new LookupService(blurringDao, patientIdSaltPrefix, patientIdSaltPostfix, auditLogger, allowedClientSystemNames);
    }

    @Test
    public void getActiveBlurrings_WithSundhedsFaglig_ThrowsException() {
        // Arrange
        String patientId = "0101019999";
        IdsasActor sundhedsFaglig = IdsasActor.createSundhedsFaglig("5678", "CVR");

        // Act
        IdsasServiceException e = assertThrows(IdsasServiceException.class, () -> lookupService.getActiveBlurrings(sundhedsFaglig, patientId, PersonIdType.CPR));

        // Assert
        assertEquals(ErrorMessages.userTypeNotSupported(IdsasActor.IdsasActorType.SUNDHEDSFAGLIG.name()), e.getMessage());
    }

    @Test()
    public void getActiveBlurrings_WithSystemUser_ReturnsActiveBlurringsAndAuditLogs() throws IdsasServiceException {
        // Arrange
        String patientId = "0101019999";
        String hashedPatientId = HashingUtil.createPatientIdHash(patientId, "123", "456");
        String patientIdType = "CPR";
        IdsasActor systemBruger = IdsasActor.createSystem("5678", "CVR", allowedClientSystemNames[0], "");
        String blurredOrganisation = "testOrg";
        Mockito.when(blurringDao.getActiveBlurrings(hashedPatientId, patientIdType))
                .thenReturn(Collections.singletonList(new BlurringEntity(null, null, blurredOrganisation, "CVR", null)));

        // Act
        List<OrganizationIdDto> activeBlurrings = lookupService.getActiveBlurrings(systemBruger, patientId, PersonIdType.CPR);

        // Assert
        assertEquals(1, activeBlurrings.size());
        assertEquals(blurredOrganisation, activeBlurrings.get(0).id);
        assertEquals(OrgIdType.CVR, activeBlurrings.get(0).idType);
        Mockito.verify(auditLogger).logGetBlurredOrganisations(patientId, patientIdType);
    }

    @Test()
    public void getActiveBlurrings_WithSystemUserWithoutAllowedClientSystemName_ThrowsException() {
        // Arrange
        String patientId = "0101019999";
        IdsasActor systemBrugerNoAccess = IdsasActor.createSystem("5678", "CVR", "notAllowedClientSystemName", "");

        // Act
        IdsasServiceException e = assertThrows(IdsasServiceException.class, () -> lookupService.getActiveBlurrings(systemBrugerNoAccess, patientId, PersonIdType.CPR));

        // Assert
        String allNames = Arrays.stream(allowedClientSystemNames).reduce((s1, s2) -> s1 + ", " + s2).orElse("");
        assertEquals(ErrorMessages.clientSystemNameError(systemBrugerNoAccess.getClientSystemName(), allNames), e.getMessage());
    }

    @Test()
    public void getActiveBlurrings_WithSystemUserThatMatchesSecondAllowedClientSystemName_ThrowsException() throws IdsasServiceException {
        // Arrange
        String patientId = "0101019999";
        IdsasActor systemBrugerNoAccess = IdsasActor.createSystem("5678", "CVR", "TestSystem2", "");

        // Act
        List<OrganizationIdDto> activeBlurrings = lookupService.getActiveBlurrings(systemBrugerNoAccess, patientId, PersonIdType.CPR);

        // Assert
        assertNotNull(activeBlurrings);
    }
}
