package dk.nsp.idsas.service;

import dk.nsp.idsas.dao.OrgBlurringDao;
import dk.nsp.idsas.dao.WhitelistDao;
import dk.nsp.idsas.dao.entity.OrgBlurringEntity;
import dk.nsp.idsas.dto.OrgIdType;
import dk.nsp.idsas.dto.OrganizationIdDto;
import dk.nsp.idsas.integrations.sores.SoresClient;
import dk.nsp.idsas.integrations.sores.SoresClientException;
import dk.nsp.idsas.security.IdsasActor;
import dk.nsp.idsas.util.ErrorMessages;
import dk.nsp.idsas.util.WhitelistType;
import org.junit.Before;
import org.junit.Test;
import org.mockito.Mockito;
import org.springframework.dao.DataAccessException;

import java.util.ArrayList;
import java.util.List;

import static org.junit.Assert.*;
import static org.mockito.Mockito.verify;
import static org.mockito.Mockito.when;

public class OrgBlurringServiceTest {
    private SoresClient soresClient;
    private OrgBlurringDao orgBlurringDao;
    private OrgBlurringService orgBlurringService;
    private WhitelistDao whiteListDao;

    @Before
    public void setUp() {
        soresClient = Mockito.mock(SoresClient.class);
        orgBlurringDao = Mockito.mock(OrgBlurringDao.class);
        whiteListDao = Mockito.mock(WhitelistDao.class);
        String[] allowedClientSystemNames = new String[]{"system1", "system2"};
        orgBlurringService = new OrgBlurringService(whiteListDao, orgBlurringDao, soresClient, allowedClientSystemNames);
    }

    @Test
    public void createOrgBlurring_sorExistsRegistrationDoesNot_noErrors() throws IdsasServiceException, SoresClientException {
        // Act
        String sor = "12345678";
        String cvr = "123";

        when(soresClient.getCvrNumberBySor(sor)).thenReturn(cvr);
        when(orgBlurringDao.orgBlurringExists("123", "SOR")).thenReturn(false);
        when(whiteListDao.isWhitelisted(cvr, WhitelistType.BLURRING)).thenReturn(true);

        // Act
        orgBlurringService.createOrgBlurring(cvr, IdsasActor.IdsasActorType.SYSTEM, sor, OrgIdType.SOR);

        // Assert
        verify(orgBlurringDao).orgBlurringExists(sor, "SOR");
        verify(orgBlurringDao).insert(Mockito.any());
    }

    @Test
    public void createOrgBlurring_registrationAlreadyExists_shouldFail() throws SoresClientException {
        final String sor = "123";
        final String cvr = "12345678";

        // Arrange
        when(orgBlurringDao.orgBlurringExists(sor, "SOR")).thenReturn(true);
        when(whiteListDao.isWhitelisted(cvr, WhitelistType.BLURRING)).thenReturn(true);
        when(soresClient.getCvrNumberBySor(sor)).thenReturn(cvr);

        // Act
        IdsasServiceException exception = assertThrows(IdsasServiceException.class, () -> {
            orgBlurringService.createOrgBlurring(cvr, IdsasActor.IdsasActorType.SYSTEM, sor, OrgIdType.SOR);
        });

        // Assert
        assertEquals(ErrorMessages.orgBlurringAlreadyExists(), exception.getMessage());
    }

    @Test
    public void createOrgBlurring_notWhiteListed_shouldFail() {
        // Arrange
        when(whiteListDao.isWhitelisted("12345678", WhitelistType.BLURRING)).thenReturn(false);

        // Act
        IdsasServiceException exception = assertThrows(IdsasServiceException.class, () -> {
            orgBlurringService.createOrgBlurring("12345678", IdsasActor.IdsasActorType.SYSTEM, "123", OrgIdType.SOR);
        });

        // Assert
        assertEquals(ErrorMessages.cvrWhitelistError("12345678", WhitelistType.BLURRING), exception.getMessage());
    }

    @Test
    public void listOrgBlurringsForCVR_shouldReturnList() throws IdsasServiceException {
        // Arrange
        OrgBlurringEntity orgBlurringEntity = new OrgBlurringEntity();
        orgBlurringEntity.setOrganisationId("123");
        orgBlurringEntity.setOrganisationIdType("SOR");
        List<OrgBlurringEntity> orgBlurringEntities = new ArrayList<>();
        orgBlurringEntities.add(orgBlurringEntity);
        when(orgBlurringDao.getAllOrgBlurrings("123")).thenReturn(orgBlurringEntities);

        // Act
        List<OrganizationIdDto> result = orgBlurringService.listOrgBlurringsForCVR("123", IdsasActor.IdsasActorType.SYSTEM);

        // Assert
        assertEquals(1, result.size());
        assertEquals("123", result.get(0).id);
    }

    @Test
    public void listOrgBlurringsForCVR_shouldReturnEmptyList() throws IdsasServiceException {
        // Arrange
        when(orgBlurringDao.getAllOrgBlurrings("123")).thenReturn(new ArrayList<>());

        // Act
        List<OrganizationIdDto> result = orgBlurringService.listOrgBlurringsForCVR("123", IdsasActor.IdsasActorType.SYSTEM);

        // Assert
        assertEquals(0, result.size());
    }

    @Test
    public void listOrgBlurringsForCVR_shouldThrowException() {
        // Arrange
        when(orgBlurringDao.getAllOrgBlurrings("123")).thenThrow(new DataAccessException("fejl") {
            @Override
            public String getMessage() {
                return super.getMessage();
            }
        });

        // Act
        IdsasServiceException exception = assertThrows(IdsasServiceException.class, () -> {
            orgBlurringService.listOrgBlurringsForCVR("123", IdsasActor.IdsasActorType.SYSTEM);
        });

        // Assert
        assertEquals(ErrorMessages.daoAccessError(), exception.getMessage());
    }

    @Test
    public void listAllActiveOrgBlurrings_shouldReturnList() throws IdsasServiceException {
        // Arrange
        OrgBlurringEntity orgBlurringEntity = new OrgBlurringEntity();
        orgBlurringEntity.setOrganisationId("123");
        orgBlurringEntity.setOrganisationIdType("SOR");
        List<OrgBlurringEntity> orgBlurringEntities = new ArrayList<>();
        orgBlurringEntities.add(orgBlurringEntity);
        when(orgBlurringDao.getAllActiveOrgBlurrings()).thenReturn(orgBlurringEntities);

        // Act
        List<OrganizationIdDto> result = orgBlurringService.listAllActiveOrgBlurrings("system1", IdsasActor.IdsasActorType.SYSTEM);

        // Assert
        assertEquals(1, result.size());
        assertEquals("123", result.get(0).id);
    }

    @Test
    public void listAllActiveOrgBlurrings_shouldReturnEmptyList() throws IdsasServiceException {
        // Arrange
        when(orgBlurringDao.getAllActiveOrgBlurrings()).thenReturn(new ArrayList<>());

        // Act
        List<OrganizationIdDto> result = orgBlurringService.listAllActiveOrgBlurrings("system1", IdsasActor.IdsasActorType.SYSTEM);

        // Assert
        assertEquals(0, result.size());
    }

    @Test
    public void listAllActiveOrgBlurrings_shouldThrowException() {
        // Arrange
        when(orgBlurringDao.getAllActiveOrgBlurrings()).thenThrow(new DataAccessException("fejl") {
            @Override
            public String getMessage() {
                return super.getMessage();
            }
        });

        // Act
        IdsasServiceException exception = assertThrows(IdsasServiceException.class, () -> {
            orgBlurringService.listAllActiveOrgBlurrings("system1", IdsasActor.IdsasActorType.SYSTEM);
        });

        // Assert
        assertEquals(ErrorMessages.daoAccessError(), exception.getMessage());
    }

    @Test
    public void listAllActiveOrgBlurrings_notAllowedClientSystemName_shouldThrowException() {
        // Act
        IdsasServiceException exception = assertThrows(IdsasServiceException.class, () -> {
            orgBlurringService.listAllActiveOrgBlurrings("system3", IdsasActor.IdsasActorType.SYSTEM);
        });

        // Assert
        assertTrue(exception.getMessage().contains("system1"));
        assertTrue(exception.getMessage().contains("system2"));
        assertTrue(exception.getMessage().contains("system3"));
    }

    @Test
    public void removeOrgBlurring_shouldRemove() throws IdsasServiceException {
        // Arrange
        String cvr = "123";
        String sor = "12345678";
        OrgBlurringEntity orgBlurringEntity = new OrgBlurringEntity();
        orgBlurringEntity.setOrganisationId(sor);
        orgBlurringEntity.setOrganisationIdType("SOR");
        orgBlurringEntity.setCreatorCvr(cvr);
        List<OrgBlurringEntity> orgBlurringEntities = new ArrayList<>();
        orgBlurringEntities.add(orgBlurringEntity);
        when(orgBlurringDao.getActiveOrgBlurrings(sor, "SOR")).thenReturn(orgBlurringEntities);

        // Act
        orgBlurringService.removeOrgBlurring(cvr, sor, OrgIdType.SOR);

        // Assert
        verify(orgBlurringDao).removeOrgBlurring(sor, "SOR");
    }

    @Test
    public void removeOrgBlurring_wrongCreatorCvr_throwsException() {
        // Arrange
        String cvr = "123";
        String sor = "12345678";
        OrgBlurringEntity orgBlurringEntity = new OrgBlurringEntity();
        orgBlurringEntity.setOrganisationId(sor);
        orgBlurringEntity.setOrganisationIdType("SOR");
        orgBlurringEntity.setCreatorCvr("456");
        List<OrgBlurringEntity> orgBlurringEntities = new ArrayList<>();
        orgBlurringEntities.add(orgBlurringEntity);
        when(orgBlurringDao.getActiveOrgBlurrings(sor, "SOR")).thenReturn(orgBlurringEntities);

        // Act
        IdsasServiceException exception = assertThrows(IdsasServiceException.class,
                () -> orgBlurringService.removeOrgBlurring(cvr, sor, OrgIdType.SOR));

        // Assert
        assertEquals(ErrorMessages.orgBlurringNotCreatedByOrg(), exception.getMessage());
    }

    @Test
    public void removeOrgBlurring_noOrgBlurring_throwsException() {
        // Arrange
        String cvr = "123";
        String sor = "12345678";
        when(orgBlurringDao.getActiveOrgBlurrings(sor, "SOR")).thenReturn(new ArrayList<>());

        // Act
        IdsasServiceException exception = assertThrows(IdsasServiceException.class,
                () -> orgBlurringService.removeOrgBlurring(cvr, sor, OrgIdType.SOR));

        // Assert
        assertEquals(ErrorMessages.orgBlurringDoesNotExist(), exception.getMessage());
    }

    @Test
    public void removeOrgBlurring_daoException_throwsException() {
        // Arrange
        String cvr = "123";
        String sor = "12345678";
        OrgBlurringEntity orgBlurringEntity = new OrgBlurringEntity();
        orgBlurringEntity.setOrganisationId(sor);
        orgBlurringEntity.setOrganisationIdType("SOR");
        orgBlurringEntity.setCreatorCvr(cvr);
        List<OrgBlurringEntity> orgBlurringEntities = new ArrayList<>();
        orgBlurringEntities.add(orgBlurringEntity);
        when(orgBlurringDao.getActiveOrgBlurrings(sor, "SOR")).thenReturn(orgBlurringEntities);
        Mockito.doThrow(new DataAccessException("fejl") {
            @Override
            public String getMessage() {
                return super.getMessage();
            }
        }).when(orgBlurringDao).removeOrgBlurring(sor, "SOR");

        // Act
        IdsasServiceException exception = assertThrows(IdsasServiceException.class,
                () -> orgBlurringService.removeOrgBlurring(cvr, sor, OrgIdType.SOR));

        // Assert
        assertEquals(ErrorMessages.daoAccessError(), exception.getMessage());
    }

}
