package dk.nsp.idsas.service;

import dk.nsp.idsas.dao.SaltDao;
import dk.nsp.idsas.security.IdsasActor;
import dk.nsp.idsas.util.AuditLogger;
import dk.nsp.idsas.util.ErrorMessages;
import org.junit.Before;
import org.junit.Test;
import org.mockito.Mockito;

import java.security.SecureRandom;
import java.util.Arrays;

import static org.junit.Assert.*;
import static org.mockito.ArgumentMatchers.any;

public class SaltServiceTest {

    private SaltDao saltDao;
    private SaltService saltService;
    private AuditLogger auditLogger;
    private SecureRandom secureRandom;
    private final String[] allowedClientSystemNames = {"itSystemName1", "itSystemName2"};

    @Before
    public void setUp() {
        saltDao = Mockito.mock(SaltDao.class);
        auditLogger = Mockito.mock(AuditLogger.class);
        secureRandom = Mockito.mock(SecureRandom.class);
        saltService = new SaltService(saltDao, auditLogger, secureRandom, allowedClientSystemNames);
    }

    @Test
    public void getCurrentSalt_WithNonSystemUser_ThrowsException() {
        // Arrange
        IdsasActor sundhedsFagligActor = IdsasActor.createSundhedsFaglig("", "");

        // Act
        IdsasServiceException e = assertThrows(IdsasServiceException.class, () -> saltService.getCurrentSalt(sundhedsFagligActor));


        assertEquals(ErrorMessages.userTypeNotSupported(sundhedsFagligActor.getActorType().name()), e.getMessage());
    }

    @Test
    public void getCurrentSalt_WhereSystemNameIsNotAllowed_ThrowsException() {
        // Arrange
        IdsasActor systemActor = IdsasActor.createSystem("", "", "", "");

        // Act
        IdsasServiceException e = assertThrows(IdsasServiceException.class, () -> saltService.getCurrentSalt(systemActor));

        // Assert
        assertEquals(ErrorMessages.clientSystemNameError(
                systemActor.getClientSystemName(),
                Arrays.stream(allowedClientSystemNames).reduce((a, b) -> a + ", " + b).orElse("")),
                e.getMessage());
    }

    @Test
    public void getCurrentSalt_WhereSystemNameIsAllowed_ReturnsSalt() throws IdsasServiceException {
        // Arrange
        IdsasActor systemActor = IdsasActor.createSystem("", "", allowedClientSystemNames[0], "");

        // Act
        byte[] salt = saltService.getCurrentSalt(systemActor);

        // Assert
        Mockito.verify(auditLogger).logGetCurrentSalt(systemActor.getClientKey(), "cvr", salt);
        Mockito.verify(saltDao).getLatestSalt();
    }

    @Test
    public void getCurrentSalt_WhereSecondSystemNameIsAllowed_ReturnsSalt() throws IdsasServiceException {
        // Arrange
        IdsasActor systemActor = IdsasActor.createSystem("", "", allowedClientSystemNames[1], "");

        // Act
        byte[] salt = saltService.getCurrentSalt(systemActor);

        // Assert
        Mockito.verify(auditLogger).logGetCurrentSalt(systemActor.getClientKey(), "cvr", salt);
        Mockito.verify(saltDao).getLatestSalt();
    }

    @Test
    public void renewSalt_insertsSecureRandomBytesAsSalt() {
        // Arrange
        byte[] newSalt = {1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 12, 13, 14, 15, 16};
        Mockito.doAnswer(func -> {
            byte[] bytes = func.getArgument(0);
            System.arraycopy(newSalt, 0, bytes, 0, newSalt.length);
            return null;
        } ).when(secureRandom).nextBytes(any());

        // Act
        saltService.renewSalt();

        // Assert
        Mockito.verify(saltDao).insert(newSalt);
    }
}